/*
STEREO CAMERA RIG
v 1.6.2

Author: David Shelton
Version: 3dsmax p2010 (probably older versions too)

new Features:
- everything is animatable
- switch between parallel and convergent
- accurate floating frame and floating window
- lots of output types (optimized anaglyph 2 versions)
- rendering of stereo image sequences
- viewport preview (rightclick)

to do
- unlimited cameras in rig
- stacked output for multiple cameras
- volume statistics
- compatibility down to max9
- Plugin
- set Layout to this camera
- nice icons :-)
- saveable presets
- speed optimizations

Creates rig with two parrallel cameras and a couple of controls. You can adjust the focus, interoccular distance and some other stuff. You can also utilize the floating frame.
Type error: Call needs function or class, got:undefined
Help:m
Go to www.davidshelton.de/blog/ and look for the stereo rig download. There you will find the newest version of the rig and proper instructions.


*/

/****************************************************************************************************************************/
/*   FUNCTIONS
/****************************************************************************************************************************/
/*macroScript createstereoCam
Category: "3dhippie"
toolTip:"create stereoCam"
buttontext: "create stereoCam"	
( */	

	global origfov, origrendtimetype, origrendstart, origrendend, origrendnth, origrendfilenumberbase, origrendPickupframes, orig_layout, \
	orig_type, orig_cam, dwidth, wascancelled, stereocam_settings_rollout, stereoprogress_rollout

	global calculateRatio, changeRangesColor, toggleFrame, toggleRanges, render_sequence, render_eachframe, \
	restore_originalvalues, save_originalvalues, getframesequence, getZeros, rendPickupAsArray, delegate_render, \
	render_cam04, render_cam02, render_stereo, preview_last, render_preview, render_viewport_preview, \
	postrender_fn, process_images, setScreenPreset
	
	persistent global allStereoCameras
	
	global store_and_comp, stored_left_frames, stored_right_frames
	
	
	fn store_and_comp =
	(
		
		a = rendoutputfilename
		
		f = ((((callbacks.notificationParam())[13]) as integer)/TicksPerFrame) as string
		if rendtimetype > 1 then str = (getfilenamepath a)+(getfilenamefile a)+(getZeros (f))+(getfilenametype a)
			else str = (getfilenamepath a)+(getfilenamefile a)+(getfilenametype a)
		str = substitutestring str "\\" "/"	
		RightImg = copy str
		leftImg = substituteString (copy str) "right" "left"
		FilePath = substituteString (copy str) "right" "stereo"
		
		
		stereoimage = process_images (openBitMap leftImg) (getlastrenderedimage()) FilePath
		save stereoimage
		close stereoimage
		framecounter += 1
	)
	
	
	fn store_and_comp_vray =
	(
		
		a = rendoutputfilename
		
		f = ((((callbacks.notificationParam())[13]) as integer)/TicksPerFrame) as string
		if rendtimetype > 1 then str = (getfilenamepath a)+(getfilenamefile a)+(getZeros (f))+(getfilenametype a)
			else str = (getfilenamepath a)+(getfilenamefile a)+(getfilenametype a)
		str = substitutestring str "\\" "/"	
		FilePath = substituteString (copy str) "right" "stereo"
		
		VrayImage = getlastrenderedimage()
		leftimage = bitmap renderwidth renderheight
		pastebitmap VrayImage leftimage [0,0] [0,0] type:#paste
		rightimage = bitmap renderwidth renderheight
		pastebitmap VrayImage rightimage [renderwidth,0] [0,0] type:#paste
		stereoimage = (process_images leftImage rightImage Filepath)	
		save stereoimage
		close stereoimage
		framecounter += 1
	)
	
	fn remove_stereo_callbacks=
	(
		callbacks.removescripts #postRenderFrame id:#3dhippie 
		callbacks.removescripts #postRender id:#3dhippie 
	)

	
	fn RemoveCamera = -- clean up scene if camera was deleted
	(
		
		try 
		(
			if (callbacks.notificationParam()).modifiers[#stereocam_settings] != undefined  then-- it's a stereoCam
			(
				theCam = (callbacks.notificationParam())
				CamNodes = theCam.modifiers[#stereocam_settings].camera_nodes
				HelperNodes = theCam.modifiers[#stereocam_settings].helper_nodes
				
				for x in CamNodes do
					try delete (getnodebyname(x)) catch()
				for x in HelperNodes do
					try delete (getnodebyname(x)) catch()
				
				deleteItem allStereoCameras (findItem allStereoCameras theCam)
			)
					
		) catch( print (getCurrentException()) )
	
	)
	callbacks.removescripts #NodePreDelete id:#3dhippie 
	callbacks.addscript #NodePreDelete "RemoveCamera()" id:#3dhippie 
	
	fn RenameCamera = -- clean up scene if camera was deleted
	(
		
		try 
		(
			if (callbacks.notificationParam())[3].modifiers[#stereocam_settings] != undefined  then-- it's a stereoCam
			(
				oldname = (callbacks.notificationParam())[1]
				newname = (callbacks.notificationParam())[2]
				theCam = (callbacks.notificationParam())[3]
				print newname
				print oldname.count
					
				CamNodes = theCam.modifiers[#stereocam_settings].camera_nodes
				HelperNodes = theCam.modifiers[#stereocam_settings].helper_nodes
				
				for x = 1 to CamNodes.count do
				(
					str = copy ((getnodebyname(CamNodes[x])).name) 
					nstr = (replace str 1 oldname.count newname)
					(getnodebyname(CamNodes[x])).name = nstr
					theCam.modifiers[#stereocam_settings].stereoCam_Controls.camera_nodes[x] = nstr
				)
				for x = 1 to HelperNodes.count do
				(
					str = copy ((getnodebyname(HelperNodes[x])).name) 
					nstr = (replace str 1 oldname.count newname)
					(getnodebyname(HelperNodes[x])).name = nstr
					theCam.modifiers[#stereocam_settings].stereoCam_Controls.helper_nodes[x] = nstr
				)
				
			)
					
		) catch( print (getCurrentException()) )
	
	)
	callbacks.removescripts #nodeNameSet id:#3dhippie 
	callbacks.addscript #nodeNameSet "RenameCamera()" id:#3dhippie 

	
	fn updateAllControllers =
	(
		try 
		(
			for scam in allStereoCameras do
			(
				scam.modifiers[#stereocam_settings].bracketsize.controller.update()
				(getNodebyName(scam.modifiers[#stereocam_settings].helper_nodes[1])).position.controller.XPosition.controller.update
				(getNodebyName(scam.modifiers[#stereocam_settings].helper_nodes[1])).length.controller.update
				(getNodebyName(scam.modifiers[#stereocam_settings].helper_nodes[1])).width.controller.update
					
				(getNodebyName(scam.modifiers[#stereocam_settings].helper_nodes[2])).position.controller.XPosition.controller.update
				(getNodebyName(scam.modifiers[#stereocam_settings].helper_nodes[2])).length.controller.update
				(getNodebyName(scam.modifiers[#stereocam_settings].helper_nodes[2])).width.controller.update	
					
			)
		)
		catch()
	)
	
	callbacks.removescripts #renderParamsChanged id:#3dhippie 
	callbacks.addScript #renderParamsChanged "updateallcontrollers()" id:#3dhippie
	
	fn postrender_fn = 
	(
		print ((callbacks.notificationParam())[13])

	)



	

fn process_images leftImg rightImg filepath fromviewport:false =
(
	
		
	left = copy leftImg
	right = copy rightImg
	crop_amount = 0
	
	
	p = (left.width*left.height)/100
	
	if filepath != undefined then newImage = bitmap right.width right.height filename:filepath else newImage = bitmap right.width right.height 
	if $.modifiers[#stereocam_settings].previewtype < 6 then pastebitmap left newImage [0,0] [0,0] type:#paste
	
	local gamma_correct,anaglyph_fn,anaglyph_corrected_fast_fn, anaglyph_corrected_fn, anaglyph_monochrome_fn, polarized_fn
	
	if $.modifiers[#stereocam_settings].previewtype < 6 then 
	(
		rollout stereoprogress_rollout "Processing two Images"
		(
			progressBar prg_process value:10 color:blue
		)
			
		createDialog stereoprogress_rollout 400 60
	)
	
	if fromviewport then --calculate offset
	(
		--fwidth = (left.width*(($.modifiers[#stereocam_settings].camdist/(2*$.modifiers[#stereocam_settings].focus*tan($.modifiers[#stereocam_settings].fov/2)))+1))
		crop_amount = 27	
	)
	
	fn gamma_correct cl_channel gamma:1.5 =	
	(
		res = ( 255 * (pow  cl_channel  (1/gamma) ))/( pow 255 ( 1 / gamma ) )
		res
	)
		
	fn anaglyph_fn color1 pixel1 color2 pixel2 = 
	(
		if (mod pixel1.x renderwidth) == 0 do (stereoprogress_rollout.prg_process.value = pixel1.y/(renderheight/100.0))
		res = color1 --the result will contain the Green and Blue of b2
		res.r = color2.r --but the R from b1
		res 
	)

	fn anaglyph_corrected_fast_fn color1 pixel1 color2 pixel2 = -- a much faster but less beautiful version of the optimized anaglyph algorithm
	(
		if (mod pixel1.x renderwidth) == 0 do (stereoprogress_rollout.prg_process.value = pixel1.y/(renderheight/100.0))
		res = color1
		res.r = (color2.g*0.7)+(color2.b*0.3)
		res
	)

	fn anaglyph_corrected_fn color1 pixel1 color2 pixel2 =  -- optimized anaglyph using the algorithm from 3dtv. Thanks to Peter Wimmer.
	(
		--stereoprogress_rollout.prg_process.value = stereoprogress_rollout.prg_process.value+p
		if (mod pixel1.x renderwidth) == 0 do (stereoprogress_rollout.prg_process.value = pixel1.y/(renderheight/100.0))
		f1 = 0.45
		f2 = 0.25
		g = 1.5
		res = color1
		
		g1_ = color2.g + (f1 * amax #(0 ,(color2.r-color2.g)))
		b1_ = color2.b + (f2 * amax #(0 ,(color2.r-color2.b)))
			
		g2_ = color1.g + (f1 * amax #(0 ,(color1.r-color1.g)))
		b2_ = color1.b + (f2 * amax #(0 ,(color1.r-color1.b)))
		
		ra_ = (191 * g1_ + 64 * b1_) / 255

		res = (color ((255 * (pow  ra_  (1/1.5) ))/( pow 255 ( 1 / 1.5 ) ))  g2_ b2_ color1.a)
		
		--res.g = g2_
		--res.b = b2_
			
		--res	
		
		
	)

	fn anaglyph_monochrome_fn color1 pixel1 color2 pixel2 = 
	(
		if (mod pixel1.x renderwidth) == 0 do (stereoprogress_rollout.prg_process.value = pixel1.y/(renderheight/100.0))
		res = color2 --the result will contain the Green and Blue of b2
		res.r = (color2.r*0.299)+(color2.g*0.587)+(color2.b*0.114)  
		res.g = (color1.r*0.299)+(color1.g*0.587)+(color1.b*0.114) 
		res.b = res.g
		res
	)

	fn polarized_fn color1 pixel1 color2 pixel2 = 
	(
		if (mod pixel1.x renderwidth) == 0 do (stereoprogress_rollout.prg_process.value = pixel1.y/(renderheight/100.0))
		if ((mod pixel1.y 2) == 0) then (res = color1) else (res = color2)
		res
	)	
	
	--fn getmax val1 val2 = ( if val1 >= val2 then (val1) else (val2))
		
	--stereoprogress_rollout.prg_process.value = 20
		
	--#("anaglyph (red/cyan)", "monochrome anaglyph","optimized anaglyph (fast)","optimized anaglyph (best)","polarized (interlaced)", "side by side")
	
		
	
	case $.modifiers[#stereocam_settings].previewtype of
	(
		1:	
		(	
			pastebitmap right newImage [0,0] [0,0] type:#function function:anaglyph_fn--anaglyph with no correction
			stereoprogress_rollout.prg_process.value = 100
			destroydialog stereoprogress_rollout
			
		)
		2:	
		(
			pastebitmap right newImage [0,0] [0,0] type:#function function:anaglyph_monochrome_fn --anaglyph with correction
			stereoprogress_rollout.prg_process.value = 100
			destroydialog stereoprogress_rollout
			
		)
		3:	
		( 
			pastebitmap right newImage [0,0] [0,0] type:#function function:anaglyph_corrected_fast_fn --polarized
			stereoprogress_rollout.prg_process.value = 100
			destroydialog stereoprogress_rollout
			
		)
		4:	
		(
			pastebitmap right newImage [0,0] [0,0] type:#function function:anaglyph_corrected_fn --polarized
			stereoprogress_rollout.prg_process.value = 100
			destroydialog stereoprogress_rollout
			
			
		)
		5:
		(
			
			pastebitmap right newImage [0,0] [0,0] type:#function function:polarized_fn --monochrome anaglyph
			stereoprogress_rollout.prg_process.value = 100
			destroydialog stereoprogress_rollout
			
		)
		6:	
		(
			if filepath != undefined then newImage = bitmap (right.width*2) right.height filename:filepath else newImage = bitmap (right.width*2) right.height
			pastebitmap left newImage [0,0] [0,0] type:#paste
			pastebitmap right newImage [0,0] [right.width,0] type:#paste
			--stereoprogress_rollout.prg_process.value = 100
			--destroydialog stereoprogress_rollout
			
		)
		7:	
		(
			if filepath != undefined then newImage = bitmap (right.width*2) right.height filename:filepath else newImage = bitmap (right.width*2) right.height
			pastebitmap left newImage [0,0] [right.width,0] type:#paste
			pastebitmap right newImage [0,0] [0,0] type:#paste
			--stereoprogress_rollout.prg_process.value = 100
			--destroydialog stereoprogress_rollout
			
		)
		
		
		
	)
	
	close left
	close right
	close newimage
	return newImage
	
	
	
)

fn render_viewport_preview thecam=
(
	/*
	viewport.activeViewport = 2
	local leftImage = gw.getviewportDib()
	viewport.activeViewport = 4
	local rightImage = gw.getviewportDib()
	*/
	save_originalvalues() 
	/*maximized = false
	if viewport.numviews > 1 do 
	(
		max tool maximize
		maximized = true
	)*/
		
	shownear = (getNodebyName(thecam.helper_nodes[1])).ishidden
	showscreen = (getNodebyName(thecam.helper_nodes[2])).ishidden
	showfar = (getNodebyName(thecam.helper_nodes[3])).ishidden
	showVolume = (getNodebyName(thecam.helper_nodes[9])).ishidden
	showlookat = (getNodebyName(thecam.helper_nodes[8])).ishidden
	showcam1 = (getNodebyName(thecam.camera_nodes[1])).ishidden
	showcam2 = (getNodebyName(thecam.camera_nodes[2])).ishidden
	showcam3 = (getNodebyName(thecam.camera_nodes[3])).ishidden	
	
		
	(getNodebyName(thecam.helper_nodes[1])).ishidden = true
	(getNodebyName(thecam.helper_nodes[2])).ishidden = true
	(getNodebyName(thecam.helper_nodes[3])).ishidden = true
	(getNodebyName(thecam.helper_nodes[9])).ishidden = true
	(getNodebyName(thecam.helper_nodes[8])).ishidden = true
	(getNodebyName(thecam.camera_nodes[1])).ishidden = true
	(getNodebyName(thecam.camera_nodes[2])).ishidden = true
	(getNodebyName(thecam.camera_nodes[3])).ishidden = true	
	
	
	
	viewport.setcamera (getNodebyName(thecam.camera_nodes[1]))
	redrawviews()
	local leftImage = gw.getviewportDib()
	viewport.setcamera (getNodebyName(thecam.camera_nodes[3]))
	redrawviews()
	local rightImage = gw.getviewportDib()
	
	global stereoImage = (process_images leftImage rightImage undefined)
	display stereoImage
		
	(getNodebyName(thecam.helper_nodes[1])).ishidden = shownear
	(getNodebyName(thecam.helper_nodes[2])).ishidden = showscreen
	(getNodebyName(thecam.helper_nodes[3])).ishidden = showfar
	(getNodebyName(thecam.helper_nodes[9])).ishidden = showVolume
	(getNodebyName(thecam.helper_nodes[8])).ishidden = showlookat	
	(getNodebyName(thecam.camera_nodes[1])).ishidden = showcam1
	(getNodebyName(thecam.camera_nodes[2])).ishidden = showcam2
	(getNodebyName(thecam.camera_nodes[3])).ishidden = showcam3
	
	--if maximized do max tool maximize
	
		
	restore_originalvalues restoreviews:true
)

fn render_preview thecam =
(
	print thecam
	
		if classof (getNodebyName(thecam.camera_nodes[2])) != Vrayphysicalcamera and thecam.useVray then 
			if querybox ("The Vray Stereoscopic Plugin works best in combination with vray physical cameras. The focus will be set at a wrong distance with standard cameras. Do you want to disable the plugin?") == true
			then thecam.useVray = false else thecam.useVray = true

		renderscenedialog.close()
			
		-- save original settings
		
		
		orig_rendtimeType = rendtimetype
		orig_savefile = rendSaveFile 
		stored_left_frames = #()
		stored_right_frames = #()	
		
		rendTimeType  = 1
		rendSaveFile  = false
			
		-- do the rendering
		callbacks.removescripts #postRenderFrame id:#3dhippie 

		if thecam.useVray then
		(
			if renderers.current.output_on then messagebox "the chosen output type has no effect if vray frame buffer is active" 
			thecam.vrayplugin.enabled = true
			thecam.vrayplugin.eye_distance.controller = thecam.camdist.controller
			thecam.vrayplugin.focus_method = 2
			thecam.vrayplugin.view = 0
			thecam.vrayplugin.adjust_resolution = true
			
			save_originalvalues()
			viewport.setCamera (getNodebyName(thecam.camera_nodes[2])) --center camera
			max quick render
			restore_originalvalues restoreviews:true

			if renderers.current.output_on == false then
			(
				VrayImage = getlastrenderedimage()
				global leftimage = bitmap	renderwidth renderheight
				pastebitmap VrayImage leftimage [0,0] [0,0] type:#paste
				global rightimage = bitmap renderwidth renderheight
				pastebitmap VrayImage rightimage [renderwidth,0] [0,0] type:#paste
				global stereoimage = (process_images leftImage rightImage undefined)
				display stereoimage	
			)		
			
		)
		else
		(
			startrender = false
			if isproperty renderers.current "output_on" then 
				if renderers.current.output_on then messagebox "Please either turn off the vray frame-buffer or use the stereoscopic plugin. The vray vfb is currently not supported for stereo previews"
				else startrender = true
			else startrender = true
			
			if startrender then
			(				
				try (thecam.vrayplugin.enabled = false)catch()
				global leftImage = render_cam02 thecam
				global framecounter = 1
				global rightImage = render_cam04 thecam
				global stereoImage = (process_images leftImage rightImage undefined)
				display stereoImage
			)		
		)
		
		--restore old settings
		rendTimeType  = orig_rendtimeType
		rendSaveFile  = orig_savefile
		
	
)		

fn preview_last =
(
	if (rightImage != undefined) and (leftimage != undefined) then
	(
		if (rightimage.width == leftimage.width) and (rightimage.height == leftimage.height) then
		(
			start = timeStamp()
			global stereoImage = (process_images leftImage rightImage undefined)
			display stereoImage
			end = timeStamp()
			format "processing took % seconds\n" ((end - start) / 1000.0)
		)
	)
)

fn render_stereo thecam =
(
	stored_left_frames = #()
	stored_right_frames = #()	
	callbacks.removescripts #postRenderFrame id:#3dhippie 
	if thecam.useVray and thecam.vrayplugin != undefined then
	(
		renderscenedialog.close()
		save_originalvalues()
		renderwidth = renderwidth*2
		--global stereoimage =
		--if renderers.current.output_on then messagebox "you cannot comp each frame when vray frame buffer is on" 
		thecam.vrayplugin.enabled = true
		thecam.vrayplugin.eye_distance.controller = thecam.camdist.controller
		thecam.vrayplugin.focus_method = 2
		thecam.vrayplugin.view = 0
		thecam.vrayplugin.adjust_resolution =true
		
		
		viewport.setCamera (getNodebyName(thecam.camera_nodes[2])) --center camera
		--callbacks.addscript #postRenderFrame "store_and_comp_vray()" id:#3dhippie 
		local a = rendoutputfilename
		local FilePath = (getfilenamepath a)+"stereo_"+(thecam.name)+"_"+(getfilenamefile a)+"_"+(getfilenametype a) 
		rendoutputfilename = FilePath	
			
		global framecounter = 1		
		max quick render
		restore_originalvalues restoreviews:true
		callbacks.removescripts #postRenderFrame id:#3dhippie 
		rendoutputfilename = a
		
		
	)
	else --no vray plugin in use
	if ((global leftImage = render_cam02 thecam) != undefined) do 
	(
		if thecam.compframes then
		(
			callbacks.removescripts #postRenderFrame id:#3dhippie 
			callbacks.addscript #postRenderFrame "store_and_comp()" id:#3dhippie 
			global framecounter = 1
		)
		global rightImage = render_cam04 thecam
		callbacks.removescripts #postRenderFrame id:#3dhippie 
	)
)

fn render_cam02 thecam= 
(
	if thecam.usevray and thecam.vrayplugin != undefined then
	(
		thecam.vrayplugin.enabled = true
		thecam.vrayplugin.eye_distance.controller = thecam.camdist.controller
		thecam.vrayplugin.focus_method = 2
		thecam.vrayplugin.view = 1
		thecam.vrayplugin.adjust_resolution =true
		
		centercam = getNodebyName(thecam.camera_nodes[2])
		global leftImage = delegate_render (centercam) -1
		
	)
	else
	(
		leftcam = getNodebyName(thecam.camera_nodes[1])
		if classof leftcam == VrayPhysicalCamera then leftcam.side = 1 --camera is left camera
		global leftImage = delegate_render (leftcam) -1 
		if classof leftcam == VrayPhysicalCamera then leftcam.side = 2 --camera is center camera
		return leftImage
	)
) -- left render

fn render_cam04 thecam= 
(
	if thecam.usevray and thecam.vrayplugin != undefined then
	(
		thecam.vrayplugin.enabled = true
		thecam.vrayplugin.eye_distance.controller = thecam.camdist.controller
		thecam.vrayplugin.focus_method = 2
		thecam.vrayplugin.view = 2
		thecam.vrayplugin.adjust_resolution =true
		
		centercam = getNodebyName(thecam.camera_nodes[2])
		global leftImage = delegate_render (centercam) 1
		
	)
	else
	(
		rightcam = getNodebyName(thecam.camera_nodes[3])
		if classof rightcam == VrayPhysicalCamera then rightcam.side = 3 -- tell it to be a right camera
		global rightImage = delegate_render (getNodebyName(thecam.camera_nodes[3])) 1 
		if classof rightcam == VrayPhysicalCamera then rightcam.side = 2 -- tell it to be a center camera	
		return rightImage	
	)
) --right render

fn delegate_render theCam side=  -- delegates the render function
(
	myImage = undefined
	
	start = timeStamp()
	myImage = render_sequence theCam side
	end = timeStamp()
	format "rendering took % seconds\n" ((end - start) / 1000.0)
	
	format "quick render % \n" $.modifiers[#stereocam_settings].scriptrender
	
	
	if myImage == undefined do myImage = getLastRenderedImage()
	return myImage
)

fn rendPickupAsArray =
(
	theStr="#{"+rendpickupframes +"}"
	for i = theStr.count to 1 by -1 where theStr[i] == "-" do 
		theStr=replace theStr i 1 ".."
	(execute theStr) as array
)

fn getZeros theNum = (substring "0000" 1 (4-(theStr = theNum as string).count) + theStr )

fn getframesequence =  --puts all frames defined inside the renderscenedialog into a single array. 
(
	local framesequence = #()
	
	case rendTimeType of
	(
		2: for i = ((animationrange.start as integer)/TicksPerFrame) to ((animationrange.end as integer)/TicksPerFrame) by rendNThFrame do append framesequence i 
		3: for i = rendstart to rendend by rendNThFrame do append framesequence i
		4: framesequence = rendPickupAsArray()
	)
		
	return framesequence
)

fn save_originalvalues =
(
	actionMan.executeAction 0 "40227"
	
	--origfov = $.modifiers[#stereocam_settings].fov
	origrendtimetype = rendtimetype
	origrendstart = rendstart
	origrendend = rendend
	origrendnth = rendNthframe
	origrendfilenumberbase = rendFileNumberBase
	origrendPickupframes = rendPickupframes
	dwidth = renderwidth
	
	orig_layout = viewport.getlayout()
	orig_type = viewport.getType()
	orig_cam = viewport.getCamera()
)

fn restore_originalvalues restoreviews:false =
(
	--$.modifiers[#stereocam_settings].fov = origfov
	rendtimetype = origrendtimetype
	rendstart = origrendstart
	rendend = origrendend
	rendNthframe = origrendnth
	rendFileNumberBase = origrendfilenumberbase
	rendPickupframes = origrendPickupframes
	renderwidth = dwidth
	
	if restoreviews then
	(
		viewport.setlayout orig_layout
		if orig_type != #view_camera then viewport.settype orig_type
		if orig_cam != undefined do viewport.setcamera orig_cam
	)
	actionMan.executeAction 0 "40228"
)



-- RENDERS A SEQUENCE
-- starts only one render with for a predefined framerange
-- faster, but doesn't take changes of th FOV or Camdist into accout.
	
fn render_sequence theCamera side= -- side can be -1 = left or 1 = right
(
	myImage = undefined
	renderscenedialog.close()
	save_originalvalues() 
	local a = rendoutputfilename
	if (side == -1) then --side = -1  --> left side 
		( local FilePath = (getfilenamepath a)+"left_"+($.name)+"_"+(getfilenamefile a)+"_"+(getfilenametype a) )
	else if (side == 1) then --side = -1  --> right side
		( local FilePath = (getfilenamepath a)+"right_"+($.name)+"_"+(getfilenamefile a)+"_"+(getfilenametype a) )
	
	if not $.modifiers[#stereocam_settings].scriptrender then
	(
		rendoutputfilename = FilePath
		viewport.setCamera theCamera
		max quick render
	) 
	else
	(
		rendoutputfilename = FilePath
		case rendTimeType of
		(
			1:	myImage = render camera:theCamera Outputfile:FilePath quiet:false frame:#current  --gamma:2.2
			2: myImage = render camera:theCamera Outputfile:FilePath quiet:false framerange:#active nthframe:rendNThFrame progressbar:true--gamma:2.2
			3: myImage = render camera:theCamera Outputfile:FilePath quiet:false fromframe:rendstart toframe:rendend nthframe:rendNThFrame progressbar:true--gamma:2.2
			4: messagebox "you can not use the frame features with quick render" 
		)
	)
	restore_originalvalues restoreviews:true
	rendoutputfilename = a
		
	return myImage
)	
	

	fn RotatePivotOnly obj rotation = 
	( 
		local rotValInv=inverse (rotation as quat)
		animate off in coordsys local obj.rotation*=RotValInv
		obj.objectoffsetrot*=RotValInv
		obj.objectoffsetpos*=RotValInv

	)
	
	fn setScreenPreset sel =
	(
		if sel == 2 then  --17" desktop screen screen 
		(
			$.modifiers[#stereocam_settings].screenWidth = 340 -- 34 cm 
			$.modifiers[#stereocam_settings].screenResolution = 1280  --horizontal screen resolution
			$.modifiers[#stereocam_settings].viewDist = 1000 -- 1m sitting distance
			$.modifiers[#stereocam_settings].ratio = (((340 as float)/(1280 as float)) as string)
			
		)		
		
		if sel == 6 then
		(
			$.modifiers[#stereocam_settings].screenWidth = 850 -- 34 cm 
			$.modifiers[#stereocam_settings].screenResolution = 1280  --horizontal screen resolution
			$.modifiers[#stereocam_settings].viewDist = 2500 -- 1m sitting distance
			$.modifiers[#stereocam_settings].ratio = (((850 as float)/(1280 as float)) as string)
		)	
		if sel == 4 then --Zalman Monitor
		(
			$.modifiers[#stereocam_settings].screenWidth = 475 -- 34 cm 
			$.modifiers[#stereocam_settings].screenResolution = 1650  --horizontal screen resolution
			$.modifiers[#stereocam_settings].viewDist = 1950 -- 1m sitting distance
			$.modifiers[#stereocam_settings].ratio = (((850 as float)/(1280 as float)) as string)
		)	
		-- other presets here:	
	)		
	
	
	--macroScript createStereoCam
	--Category: "3dhippie"
	--toolTip:"Create stereo camera"
	--buttontext: "Create stereo camera"
	--(
		disableSceneRedraw()	
		undo off
		(	
			
			referenceCam = undefined
			if ((classof $) == Targetcamera)or((classof $) == Freecamera)or(((classof $) == VrayPhysicalcamera)and ($.targeted)) then 
			(
				if (querybox ("Do you want to create a stereo camera at the position of the currently selected camera?")) do referenceCam = $
			) 
			
			set animate off
				
			fn get2Zeros theNum = (substring "00" 1 (2-(theStr = theNum as string).count) + theStr )
			i = 1
			while (getNodeByName ("$stereoCam"+(get2zeros i))) != undefined or (LayerManager.getLayerFromName ("stereoCam"+(get2zeros i)))!= undefined do i += 1
				
			circle1 = SplineShape pos:[0,0,0] name:("stereoCam"+(get2zeros i) )
			addNewSpline circle1
			addKnot circle1 1  #bezierCorner #curve [-4.99669,-2.15254e-006,3.43144e-006] [-4.9967,2.7571,3.43144e-006] [-4.99669,-2.75711,3.43144e-006]
			addKnot circle1 1  #bezierCorner #curve [-1.06048e-006,-4.99669,3.43144e-006] [-2.75711,-4.9967,3.43144e-006] [2.7571,-4.99669,3.43144e-006]
			addKnot circle1 1  #bezierCorner #curve [4.99669,-1.71572e-006,3.43144e-006] [4.99669,-2.75711,3.43144e-006] [4.99669,2.7571,3.43144e-006]

			addNewSpline circle1
			addKnot circle1 2  #bezierCorner #curve [-1.49731e-006,-2.15254e-006,4.9967] [-1.49731e-006,2.7571,4.9967] [-1.49731e-006,-2.75711,4.9967]
			addKnot circle1 2  #bezierCorner #curve [-1.71572e-006,-4.99669,2.7762e-006] [-1.5952e-006,-4.9967,2.75711] [-1.83624e-006,-4.99669,-2.7571]
			addKnot circle1 2  #bezierCorner #curve [-1.93413e-006,-1.71572e-006,-4.99669] [-1.93413e-006,-2.75711,-4.99669] [-1.93413e-006,2.7571,-4.99669]

			addNewSpline circle1
			addKnot circle1 3  #bezierCorner #curve [4.99669,-1.71572e-006,3.43144e-006] [4.99669,-1.5952e-006,-2.7571] [4.99669,-1.83624e-006,2.75711]
			addKnot circle1 3  #bezierCorner #curve [-1.93413e-006,-1.93413e-006,4.9967] [2.7571,-1.93413e-006,4.9967] [-2.75711,-1.93413e-006,4.9967]
			addKnot circle1 3  #bezierCorner #curve [-4.99669,-1.71572e-006,2.99461e-006] [-4.9967,-1.83624e-006,2.75711] [-4.99669,-1.5952e-006,-2.7571]
			addKnot circle1 3  #bezierCorner #curve [-1.06048e-006,-1.49731e-006,-4.99669] [-2.75711,-1.49731e-006,-4.99669] [2.7571,-1.49731e-006,-4.99669]
			close circle1 3

			updateShape circle1
			
			addModifier circle1 (XForm())
			addModifier circle1 (EmptyModifier())
			circle1.modifiers[1].name = "stereoCam_settings"
			circle1.wirecolor = [225,198,87]	
			RotatePivotOnly circle1 (eulerangles 0 0 90)	

				
			ca=attributes stereoCam_Controls (
				
				parameters general  
				(
					
					v_ffov type:#angle   
					v_fwidth type:#float
					v_NPP type:#float
					v_screenplanewidth type:#float
					v_unitsToPixels type:#float
					
				)
				
				parameters settingsP rollout:settingsR 
				(
					--ResetAll type:#integer ui:(RResetBtn)
					fov type:#angle ui:(fovSp)  default:45
					fov_offset type:#angle 
					convergence type:#float ui:(convergenceSp)
					imagewidth_offset type:#float
					focus type:#worldunits ui:(focusSp) default:198
					camdist type:#worldunits ui:(camdistSp) 
					auto type:#boolean ui:(autoCb)
					preset type:#float ui:(presetSp)
					camera_nodes type:#stringTab tabSize:4 --#(stereoCam_02.name,stereoCam_03.name,stereoCam_04.name,stereoCam_helper.name)
					helper_nodes type:#stringTab tabSize:9 --#(FarPlane.name, screenPlane.name, nearPlane.name, fsf_L.name, fsf_r.name, floatwindow.name, stereoCam_focus.name, stereoCam_lookat.name, bracket.name, ArrowHelper.name )	
					stereo_Layers type:#stringTab tabSize:0 tabSizeVariable:true
					camtype type:#integer ui:(Type_ddl) default:1
					
					base_enabled type:#boolean default:false animatable:false
					preset_enabled type:#boolean default:true animatable:false
					flatter_enabled type:#boolean default:true animatable:false
					deeper_enabled type:#boolean default:true animatable:false
					thirty_enabled type:#boolean default:true animatable:false
					lens_enabled type:#boolean default:true animatable:false
					infinity_enabled type:#boolean default:true animatable:false
				)
				
				parameters actionsP rollout:actionsR
				(
					--optimized type:#boolean ui:(optimizedCb)
					previewtype type:#integer ui:(previewtypeDdl)
					scriptrender type:#boolean ui:(scriptcb)animatable:false
					compframes type:#boolean ui:(compcb) animatable:false
					useVray type:#boolean ui:(useVraycb) animatable:false default:false 
					useVray_enable type:#boolean default:false animatable:false
					vrayplugin type:#node ui:(vrayplugin_pb) animatable:false
					vrayplugin_text type:#string default:"pick" animatable:false
					
					--on useVray_enable set val do this.actionsR.useVraycb
				)
				
				
				parameters visibilityP rollout:visibilityR
				(
					nearColor type:#color ui:(nearColorCp)
					showNear type:#boolean ui:(showNearCb) --animatable:false
					nearOpacity type:#float ui:(NearOpacitySp)
					
					screenColor type:#color ui:(screenColorCp)
					showscreen type:#boolean ui:(showScreenCb) --animatable:false
					screenOpacity type:#float ui:(ScreenOpacitySp)
					
					farColor type:#color ui:(farColorCp)
					showFar type:#boolean ui:(showFarCb) --animatable:false
					FarOpacity type:#float ui:(FarOpacitySp)
					
					volumeColor type:#color ui:(volumeColorCp)
					showVolume type:#boolean ui:(showVolumeCb) --animatable:false
					VolumeOpacity type:#float ui:(VolumeOpacitySp)
					
				)
				
				parameters rangesP rollout:rangesR
				(
					
								
					imagewidth type:#integer ui:(imagewidth_spn) default:1920
					nearLimit type:#float ui:(nearLim_spn)
					farlimit type:#float ui:(farlim_spn)
									
					bracketClipping type:#boolean ui:(brClipBoundsCb) 
					
				)
				
				parameters frameP rollout:frameR
				(
					showFrame type:#boolean ui:(showFrameCb) animatable:false
					frameColor type:#color ui:(frameColorCp)
					upRight type:#float ui:(upRightSp)  
					upLeft type:#float ui:(upLeftSp)  
					downRight type:#float ui:(downRightSp)  
					downLeft type:#float ui:(downLeftSp)  
					--borderwidth type:#float ui:(borderwidthSp)
				)
				
				
				
				
				rollout settingsR "StereoCam Controls" width:162 height:263
				(
					spinner fovSp "Field of view: " pos:[13,35] width:136 height:16 range:[0,175,45] type:#float toolTip:"sets the Field of View of your stereo camera" fieldwidth:60
					spinner focusSp "Zero Parallax" pos:[9,56] width:140 height:16 range:[0,99999,500] type:#worldunits toolTip:"Area with no visible Parallax on screen. Also referred to as screen plane, ZPS or HIT" fieldwidth:60
					spinner convergenceSp "Toe-in: " pos:[18,77] width:131 height:16 range:[0,1,0] type:#float toolTip:"0= parallel 100=convergent" fieldwidth:60
					spinner camdistSp "Interaxial: " enabled:base_enabled pos:[18,98] width:131 height:16 range:[0,99999,16] type:#worldunits toolTip:"sets the interoccular distance between your cameras. Also referred to as interaxial" fieldwidth:60
					--checkbox autoCb "Auto"  align:#right
					checkbox autoCb "Auto" pos:[7,119] width:45 height:19
					spinner presetSp "1/" pos:[64,119] enabled:preset_enabled width:84 height:16 range:[0.0001,99999,30] toolTip:"% of the distance to focus" fieldwidth:60
					GroupBox grp1 "camera seperation" pos:[7,148] width:148 height:83
					dropdownList Type_ddl "" pos:[63,4] width:86 height:21 items:#("target camera", "free camera") fieldwidth:60

					label lbl18 "Type:" pos:[25,7] width:27 height:13
					button Lens_btn "by Lens" pos:[39,199] width:49 height:23 enabled:lens_enabled 
					button infinity_btn "to infinity" pos:[90,199] width:60 height:23 enabled:infinity_enabled 
					button flatter_btn "flatter (+5)" pos:[11,170] width:69 height:23 enabled:flatter_enabled 
					button deeper_btn "deeper (-5)" pos:[83,170] width:67 height:23 enabled:deeper_enabled 
					button default_btn "30" pos:[12,199] width:25 height:23 enabled:thirty_enabled 
					
						
					
					
					
					/*(
						nearplane = (getNodebyName(this.helper_nodes[3])) 
						this.preset = (cameraFOV.FOVtoMM this.FOV)*this.focus/nearplane.pos.y
					)*/	

					on autoCb changed val do
					(
						base_enabled = not val
						camdistSp.enabled = base_enabled
						preset_enabled = val
						presetSp.enabled = preset_enabled
						flatter_enabled = val
						flatter_btn.enabled = flatter_enabled
						deeper_enabled = val
						deeper_btn.enabled = deeper_enabled
						thirty_enabled = val
						default_btn.enabled = thirty_enabled
						lens_enabled = val
						lens_btn.enabled = lens_enabled
						infinity_enabled = val
						infinity_btn.enabled = infinity_enabled
						
						
						
						if not val then this.camdist.controller[2].controller.value = this.camdist.controller[1].controller.value
						
					)
					on Type_ddl selected val do
					(
						if val == 2 then
						(
							if $.rotation.controller.count > 1 then
							(
								--disablesceneredraw()
								$.rotation.controller.delete ($.rotation.controller[#lookat].index)
								$.rotation.controller[#roll][#Y_Rotation].controller = bezier_float()
								$.rotation.controller[#roll][#Z_Rotation].controller = bezier_float()
								rotate $ (angleaxis 90 [0,0,1])
								delete (getnodebyname this.helper_nodes[8])
								--enablesceneredraw()
							)
						)
						if val == 1 then
						(
							stereoCam_lookAt = Point size:40 cross:off box:on name:(circle1.name+"_lookAt")
							paramWire.connect this.stereoCam_Controls[#focus] stereoCam_lookAt.baseObject[#Size] "focus/10"
							stereoCam_lookAt.pos = (getnodebyname circle1.modifiers[#stereocam_settings].helper_nodes[2]).pos
							stereoCam_lookAt.wirecolor = [225,198,87] -- [225,198,87]
							lk = LookAt_Constraint ()
							lk.appendTarget stereoCam_lookAt 100
							lk.viewline_length_abs = false
							lk.target_Axis = 0
							lk.upnode_world = on
							lk.upnode_ctrl = 1
							rotate $ (angleaxis -90 [0,0,1])
							new_index = circle1.rotation.controller.count+1
							circle1.rotation.controller.available.controller = lk
							circle1.rotation.controller.setname new_index "lookat"
							circle1.rotation.controller.cut circle1.rotation.controller.roll.index
							circle1.rotation.controller.paste new_index
							circle1.rotation.controller.setactive new_index
							
							lk = float_limit()
							lk.lower_limit = 0;
							lk.upper_limit = 0;
							circle1.rotation.controller[#roll].controller[2].controller = lk
							circle1.rotation.controller[#roll].controller[3].controller = lk
							circle1.modifiers[#stereocam_settings].stereoCam_Controls.helper_nodes[8] = stereoCam_lookat.name	
						)
						
					)
					on Lens_btn pressed do
						this.preset = cameraFOV.FOVtoMM this.FOV
					on infinity_btn pressed do
					(
						this.preset = cameraFOV.FOVtoMM this.FOV	
						disparity = (this.farlimit*renderwidth/this.imagewidth)/this.v_unitsToPixels
						this.preset = (this.focus*3.40E5-this.focus^2)/(disparity*3.40E5)
					)
					on flatter_btn pressed do
						this.preset += 5
					on deeper_btn pressed do
						this.preset -= 5
					on default_btn pressed do
						this.preset = 30
				)
				
				rollout actionsR "Actions" width:160 height:248
				(
					fn stereo_filt obj = classof obj == VRayStereoscopic and isproperty obj #adjust_resolution
					
					button previewBtn "Preview" pos:[8,54] width:100 height:27 toolTip:"renders a preview with the selected preview type. It's always a still image and neither left nor right image will be saved to disk."
					button previewLastBtn "again" pos:[110,54] width:42 height:27 toolTip:"the selected preview type will be applied to the last rendered stereo image pair to generate a new stereo image"	
					button stereoBtn "render stereo" pos:[8,86] width:144 height:27 toolTip:"renders the left and right camera and saves the images to disk"
					label lbl1 "render cam:" pos:[8,117] width:144 height:16
					button cam02Btn "left" pos:[8,141] width:70 height:24 toolTip:"renders the left camera and saves the images to disk"
					button cam04Btn "right" pos:[82,141] width:70 height:24 toolTip:"renders the right camera and saves the images to disk"
					dropdownList previewtypeDdl "output type" pos:[8,8] width:144 height:40 items:#("color anaglyph (red/cyan)", "mono. anaglyph (red/cyan)", "optimized anaglyph (fast)", "optimized anaglyph (best)", "polarized (interlaced)", "side by side (LR)", "side by side (RL)")
					checkbox scriptcb "script render" pos:[8,176] width:143 height:14
					checkbox compcb "composite each frame" pos:[8,198] width:143 height:14
					checkbox useVraycb "use VRay Plugin" pos:[8,220] width:98 height:14	enabled:useVray_enable			
					pickButton vrayplugin_pb "pick" pos:[105,219] text:vrayplugin_text width:49 height:20 filter:stereo_filt toolTip:"Choose the vray stereoscopic plugin. You should have only one Vray plugin in your scene. Requires Vray 1.5 SP 5"
					
					--checkbox Vray_cb "use VRay Plugin" pos:[8,220] width:143 height:14
					
					
					on previewBtn pressed do
						render_preview this
					on previewBtn rightClick do
						render_viewport_preview this
					on previewLastBtn pressed do
						preview_last()
					on stereoBtn pressed do
						render_stereo this
					on cam02Btn pressed do
						render_cam02 this
					on cam04Btn pressed do
						render_cam04 this
					
					
					on vrayplugin_pb picked obj do

					( 
						--see if the user did not cancel the picking...
						 if obj != undefined do 
						 (
							vrayplugin_text = obj.name
							vrayplugin_pb.text = vrayplugin_text
							useVray_enable = true
							useVraycb.enabled = useVray_enable
							useVraycb.checked = true
							 
						 )
					)
				)	
							
				rollout VisibilityR "Visibility" width:160 height:245
				(
					local opacrange = [0,1,0.2]
					colorPicker nearColorCp "" pos:[64,8] width:24 height:20 fieldWidth:20--width:20 height:20
					checkbox showNearCb "near" pos:[8,8] width:56 height:16 checked:true
					spinner NearOpacitySp "" pos:[95,8] width:48 height:16 range:[0,1,0.2]
					colorPicker screenColorCp "" pos:[64,32] width:24 height:20 color:(color 0 0 155) fieldWidth:20
					checkbox showScreenCb "screen" pos:[8,32] width:56 height:16 checked:true
					spinner ScreenOpacitySp "" pos:[95,32] width:48 height:16 range:[0,1,0.2]
					colorPicker farColorCp "" pos:[64,56] width:24 height:20 color:(color 0 0 155) fieldWidth:20
					checkbox showFarCb "far" pos:[8,56] width:56 height:16 checked:true
					spinner FarOpacitySp "" pos:[95,56] width:48 height:16 range:[0,1,0.2]
					colorPicker volumeColorCp "" pos:[64,80] width:24 height:20 color:(color 0 0 155) fieldWidth:20
					checkbox showVolumeCb "volume" pos:[8,80] width:56 height:16 checked:true
					spinner VolumeOpacitySp "" pos:[95,80] width:48 height:16 range:[0,1,0.2]
					
					on nearColorCp changed val do
						(getNodebyName(this.helper_nodes[3])).wirecolor = val
					on showNearCb changed val do
						(getNodebyName(this.helper_nodes[3])).ishidden = not val
					on screenColorCp changed val do
						(getNodebyName(this.helper_nodes[2])).wirecolor = val
					on showScreenCb changed val do
						(getNodebyName(this.helper_nodes[2])).ishidden = not val
					on farColorCp changed val do
						(getNodebyName(this.helper_nodes[1])).wirecolor = val
					on showFarCb changed val do
						(getNodebyName(this.helper_nodes[1])).ishidden = not val
					on volumeColorCp changed val do
						(getNodebyName(this.helper_nodes[9])).wirecolor = val
					on showVolumeCb changed val do
						(getNodebyName(this.helper_nodes[9])).ishidden = not val
				)
				
				
				
				rollout rangesR "Ranges" width:160 height:176
				(
					
					
					
					GroupBox grp4 "Parallax limits" pos:[5,64] width:150 height:71
					spinner nearLim_spn "Near (neg.)" pos:[31,85] width:100 height:16 range:[-100000,100000,20] fieldwidth:40
					spinner farlim_spn "Far (pos.)" pos:[23,109] width:108 height:16 range:[-100000,100000,15] fieldwidth:40
					label lbl11 "px" pos:[138,85] width:16 height:16
					label lbl12 "px" pos:[138,109] width:16 height:16
					spinner imagewidth_spn "final imagewidth" pos:[16,37] width:116 height:16 range:[0,100000,2000] type:#integer fieldwidth:40
					label lbl3 "px" pos:[138,37] width:16 height:16
					label lbl4 "presets:" pos:[8,13] width:61 height:16
					dropDownList presets_ddl1 "" pos:[57,8] width:95 height:21 items:#("cinema", "computerscreen")
					checkbox brClipBoundsCb "clip to bounds" pos:[5,141] width:88 height:16 enabled:true checked:true


					on brClipBoundsCb changed val do
					(
						if classof (getNodebyName(this.camera_nodes[2])) != VrayPhysicalcamera then
							(
								(getNodebyName(this.camera_nodes[1])).clipmanually = val
								(getNodebyName(this.camera_nodes[2])).clipmanually = val
								(getNodebyName(this.camera_nodes[3])).clipmanually = val
							)
							else
							(
								try
								(
									(getNodebyName(this.camera_nodes[1])).clip_on = val
									(getNodebyName(this.camera_nodes[2])).clip_on = val
									(getNodebyName(this.camera_nodes[3])).clip_on = val
								)
								catch()
							)
					)
					
					on presets_ddl1 selected val do
					(
						if val == 1 then
						(
							this.imagewidth = 1920
							this.nearlimit = 18
							this.farlimit = 20
						)
						else
						(
							this.imagewidth = 1920
							this.nearlimit = 20
							this.farlimit = 40
						)
					)
						
				)

				rollout frameR "Floating frame" width:162 height:115
				(
					--spinner borderwidthSp "Borderwidth"  range:[0,100,10] type:#float
					--checkbox FFrameChk "show floating frame" tooltip:"activates or deactivates the floating frame" align:#left6 width:147 height:19 checked:true
					checkbox showFrameCb "visible" pos:[38,7] width:53 height:19 checked:true
					colorPicker frameColorCp "" pos:[8,7] width:24 height:20 color:[0,0,0] fieldWidth:20
					spinner upRightSp "" pos:[88,54] width:66 height:16 range:[0,100,0] type:#float toolTip:"offsets the upper right corner of the floating frame"
					spinner upLeftSp "" pos:[8,54] width:66 height:16 range:[0,100,0] type:#float toolTip:"offsets the upper left corner of the floating frame"
					spinner downRightSp "" pos:[88,93] width:66 height:16 range:[0,100,0] type:#float toolTip:"offsets the lower right corner of the floating frame"
					spinner downLeftSp "" pos:[8,92] width:66 height:16 range:[0,100,0] type:#float toolTip:"offsets the lower left corner of the floating frame"
					label lbl1 "Up Left:" pos:[10,39] width:67 height:12

					
			

					fn toggleFrame =
					(
						--print 'bla' --(this.showRanges)
						if (this.showFrame ==false) then
						(
							hide (getNodebyName(this.helper_nodes[4]))
							hide (getNodebyName(this.helper_nodes[5]))
							hide (getNodebyName(this.helper_nodes[6]))							
						)	else
						(
							unhide (getNodebyName(this.helper_nodes[4]))
							unhide (getNodebyName(this.helper_nodes[5]))
							unhide (getNodebyName(this.helper_nodes[6]))	
						)
					)	
			
					label lbl2 "Up Right:" pos:[90,40] width:67 height:12
					label lbl3 "Down Left:" pos:[10,77] width:67 height:12
					label lbl4 "Down Right:" pos:[90,78] width:67 height:12
					on showFrameCb changed val do
						toggleFrame()
				)
				
				
				rollout viewports_rl "Viewport presets" width:162 height:400
				(
					button setup1_btn "3 cams + perspective" pos:[10,7] width:144 height:24
					button setup2_btn "2 cams + perspective (top)" pos:[10,36] width:144 height:24
					button setup5_btn "2 cams + persp. (bottom) " pos:[10,65] width:144 height:24
					button setup3_btn "Cross eyed free-view" pos:[10,94] width:144 height:24
					button setup4_btn "default" pos:[10,123] width:144 height:24
					
					
					
					on setup1_btn pressed do
					(
						viewport.setLayout	#layout_4hb
						viewport.activeviewport = 1
						viewport.SetRenderLevel #smoothhighlights
						viewport.settype #view_persp_user
						viewport.activeviewport = 2
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[1]))
						viewport.activeviewport = 3
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[2]))
						viewport.activeviewport = 4
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[3]))	
						
					)
					on setup2_btn pressed do
					(
						viewport.setLayout	#layout_3hb
						viewport.activeviewport = 1
						viewport.SetRenderLevel #smoothhighlights
						viewport.settype #view_persp_user
						viewport.activeviewport = 2
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[1]))
						viewport.activeviewport = 3
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[3]))
										
					)
					on setup5_btn pressed do
					(
						viewport.setLayout	#layout_3ht 
						viewport.activeviewport = 3
						viewport.SetRenderLevel #smoothhighlights
						viewport.settype #view_persp_user
						viewport.activeviewport = 1
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[1]))
						viewport.activeviewport = 2
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[3]))
										
					)
					on setup3_btn pressed do
					(
						viewport.setLayout	#layout_2v
						viewport.activeviewport = 1
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[3]))
						viewport.activeviewport = 2
						viewport.SetRenderLevel #wireframe
						viewport.setcamera (getNodebyName(this.camera_nodes[1]))
										
					)
					on setup4_btn pressed do
					(
						viewport.setLayout	#layout_4
						viewport.activeviewport = 1
						viewport.SetRenderLevel #wireframe
						viewport.settype #view_top
						viewport.activeviewport = 2
						viewport.settype #view_left
						viewport.SetRenderLevel #wireframe
						viewport.activeviewport = 3
						viewport.settype #view_front
						viewport.SetRenderLevel #wireframe
						viewport.activeviewport = 4
						viewport.settype #view_persp_user
						viewport.SetRenderLevel #smoothhighlights
					
					)
				)
				
				
				
				rollout aboutR "about"
				(
					
					label info1lbl "3dhippie_stereoCam" 
					label info2lbl "version 1.0"
					label info3lbl "by David Shelton"
					label info4lbl "www.davidshelton.de\\blog\\"
				)
				
			)
			custAttributes.add circle1.modifiers[#stereocam_settings] ca

			
				
			stereoCam_focus = Point pos:[0,198,0] size:40 cross:on box:off name:(circle1.name+"_focus")
			stereoCam_focus.parent = circle1
			RotatePivotOnly stereocam_focus (eulerangles 0 0 90)	
			setTransformLockFlags stereoCam_focus #{2..9}
			stereoCam_focus.wirecolor = [225,198,87] --[225,198,87]
			
			if ClassOf referenceCam != freecamera then
			(
				stereoCam_lookAt = Point pos:[0,200,0] size:40 cross:off box:on name:(circle1.name+"_lookAt")
				stereoCam_lookAt.wirecolor = [225,198,87] -- [225,198,87]
			)
			stereoCam_helper = ExposeTm pos:[0,0,0] isSelected:off name:(circle1.name+"_helper") centerMarker:false axistripod:false cross:false box:false
			stereoCam_helper.parent = circle1
			stereoCam_helper.exposeNode = circle1
			stereoCam_helper.useParent = off
			stereoCam_helper.localReferenceNode = stereoCam_focus
			stereoCam_helper.wirecolor = [88,144,255]--[88,144,255]
			circle1.wirecolor = [225,198,87]



			circle1.modifiers[#stereocam_settings].fov.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].fov_offset.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].imagewidth_offset.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].focus.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].convergence.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].camdist.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].auto.controller = Boolean_float ()
			circle1.modifiers[#stereocam_settings].preset.controller = bezier_float()
			
			circle1.modifiers[#stereocam_settings].UpLeft.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].UpRight.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].DownRight.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].DownLeft.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].focus = 198
			circle1.modifiers[#stereocam_settings].preset = 30
			circle1.modifiers[#stereocam_settings].fov_offset = 0
			circle1.modifiers[#stereocam_settings].imagewidth_offset = 0
			circle1.modifiers[#stereocam_settings].previewtype = 3  --standard is fast optimized anaglyph
		

			circle1.modifiers[#stereocam_settings].showNear = true
			circle1.modifiers[#stereocam_settings].nearOpacity.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].nearOpacity = 0.2
			circle1.modifiers[#stereocam_settings].nearColor.controller = color_RGB()
			circle1.modifiers[#stereocam_settings].nearColor = color 225 198 87
			
			circle1.modifiers[#stereocam_settings].showFar = true
			circle1.modifiers[#stereocam_settings].FarOpacity.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].FarOpacity = 0.2
			circle1.modifiers[#stereocam_settings].FarColor.controller = color_RGB()
			circle1.modifiers[#stereocam_settings].FarColor = color 135 6 6
			
			circle1.modifiers[#stereocam_settings].showScreen = true
			circle1.modifiers[#stereocam_settings].ScreenOpacity.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].ScreenOpacity = 0.2
			circle1.modifiers[#stereocam_settings].screenColor.controller = color_RGB()
			circle1.modifiers[#stereocam_settings].screenColor = color 0 100 0
			
			circle1.modifiers[#stereocam_settings].showVolume = true
			circle1.modifiers[#stereocam_settings].VolumeOpacity.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].VolumeOpacity = 0.2
			circle1.modifiers[#stereocam_settings].VolumeColor.controller = color_RGB()
			circle1.modifiers[#stereocam_settings].VolumeColor = color 22 76 157
			
			circle1.modifiers[#stereocam_settings].frameColor.controller = color_RGB()
			circle1.modifiers[#stereocam_settings].showFrame = false
			
			circle1.modifiers[#stereocam_settings].nearlimit.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].nearlimit = 20
			circle1.modifiers[#stereocam_settings].farlimit.controller = bezier_float()
			circle1.modifiers[#stereocam_settings].farlimit = 18
			
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.fov = 45
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist = 198/30
			--$stereoCam.modifiers[#Attribute_Holder].stereoCam_Controls.cams = 2
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.auto = true
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller = float_list()
			circle1.modifiers[#stereoCam_settings].stereoCam_Controls.camdist.controller[1].controller = float_script()
			/*param1 = stereoCam_focus.pos.controller[#Y_Position]
			param2 = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.camdist.controller[#Bezier_Float]
			paramWire.connect param1 param2 "Y_Position/30"	*/
			--camdist_script = float_script()
			circle1.modifiers[#stereoCam_settings].stereoCam_Controls.camdist.controller[1].controller.addObject "focusdist"  circle1.modifiers[#stereocam_settings].stereoCam_Controls.focus.controller
			circle1.modifiers[#stereoCam_settings].stereoCam_Controls.camdist.controller[1].controller.addObject "cam_seperation"  circle1.modifiers[#stereocam_settings].preset.controller
			circle1.modifiers[#stereoCam_settings].stereoCam_Controls.camdist.controller[1].controller.script = "focusdist.value/cam_seperation.value"
			circle1.modifiers[#stereoCam_settings].stereoCam_Controls.camdist.controller[1].controller = float_script()
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.available.controller = bezier_float()

			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.active = 2 	
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.weights[1].controller = float_script()
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.weights[1].controller.addObject "auto" circle1.modifiers[#stereocam_settings].auto.controller
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.weights[1].controller.script = "abs(auto.value)"	
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.weights[2].controller = float_script()
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.weights[2].controller.addObject "auto" circle1.modifiers[#stereocam_settings].auto.controller
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camdist.controller.weights[2].controller.script = "abs(auto.value-1)"
			
			
			try (
			offsetDisp = float_script()
			offsetDisp.addObject "camdist" circle1.modifiers[#stereocam_settings].camdist.controller
			offsetDisp.addObject "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
			offsetDisp.addObject "fov" circle1.modifiers[#stereocam_settings].fov.controller
			offsetDisp.addObject "bracketOffset" circle1.modifiers[#stereocam_settings].bracketoffset.controller
			str = "realFov = radtodeg (fov.value) \n"	
			str += "fwidth = (renderwidth*((camdist.value/(2*focus.value*tan(realFov/2)))+1)) -- the final image width pixels \n"
			str += "ffov = (2*atan((fwidth*tan(realFov/2))/renderwidth))	-- final fov \n"
			str += " screenplanewidth = (((tan(ffov/2))*(focus.value))*2) -- the actual width of the screenplane in units \n"
			str += " unitsToPixels = (fwidth/screenplanewidth) \n"
			str += " bracketOffsetDisp = bracketOffset.value/unitsToPixels \n"
			str += " bracketOffsetPos = -((camdist.value*focus.value)/(bracketOffsetDisp-camdist.value))-focus.value \n"
			offsetDisp.script = str
			circle1.modifiers[#stereocam_settings].bracketoffsetDisp.controller = offsetDisp
			) catch ()
			
			
			
			stereoCam_focus.pos.X_Position.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.focus.controller
			
			v_fwidth_fs = float_script()
			v_fwidth_fs.addObject "v_camdist" circle1.modifiers[#stereocam_settings].camdist.controller
			v_fwidth_fs.addObject "v_focus" circle1.modifiers[#stereocam_settings].focus.controller --z
			v_fwidth_fs.addObject "v_fov" circle1.modifiers[#stereocam_settings].fov.controller
			v_fwidth_fs.script = "v_fwidth = (renderwidth*((v_camdist.value/(2*v_focus.value*tan((radtodeg v_fov.value)/2)))+1)) -- the final image width pixels "
			circle1.modifiers[#stereocam_settings].v_fwidth.controller	= v_fwidth_fs
				
			v_ffov_fs = float_script()
			v_ffov_fs.addObject "v_fov" circle1.modifiers[#stereocam_settings].fov.controller
			v_ffov_fs.addObject "v_fwidth" circle1.modifiers[#stereocam_settings].v_fwidth.controller
			v_ffov_fs.script = "v_ffov = (2*atan((v_fwidth.value*tan((radtodeg v_Fov.value)/2))/renderwidth))	-- final fov "	
			circle1.modifiers[#stereocam_settings].v_ffov.controller	= v_ffov_fs
			
			v_screenplanewidth_fs = float_script()
			v_screenplanewidth_fs.addObject "v_focus" circle1.modifiers[#stereocam_settings].focus.controller --z
			v_screenplanewidth_fs.addObject "v_ffov" circle1.modifiers[#stereocam_settings].v_ffov.controller
			v_screenplanewidth_fs.script = "(((tan(v_ffov.value/2))*(v_focus.value))*2) -- the actual width of the screenplane in units "
			circle1.modifiers[#stereocam_settings].v_screenplanewidth.controller = v_screenplanewidth_fs

			v_unitsToPixels_fs = float_script()
			v_unitsToPixels_fs.addObject "v_fwidth" circle1.modifiers[#stereocam_settings].v_fwidth.controller	
			v_unitsToPixels_fs.addObject "v_screenplanewidth" circle1.modifiers[#stereocam_settings].v_screenplanewidth.controller
			v_unitsToPixels_fs.script = " v_unitsToPixels = (v_fwidth.value/v_screenplanewidth.value)"
			circle1.modifiers[#stereocam_settings].v_unitsToPixels.controller = v_unitsToPixels_fs
		
		
			select circle1
			
			/*****************************************************************************************************************************
			/*   create Ranges and Floating window
			/****************************************************************************************************************************/

				------------------------------------------
				-- DEPTHBRACKET
				------------------------------------------
				Bracket = box pos:[0,0,-25] length:50 height:50 width:50 lengthsegs:1 widthsegs:2 heightsegs:1 isSelected:off name:(circle1.name+"_bracket")
				Bracket.pivot = [0,0,0] 
				
				--Bracket = plane length:50 width:50 name:(circle1.name+"_Bracket") isSelected:off pos:[0,0,0] widthsegs:2 lengthsegs:2
				--Bracket.pivot = [0,0,0]
				Bracket.transform = circle1.transform
				
				rotate Bracket (angleaxis -90 [0,1,0])
				Bracket.parent = circle1
				--Bracket.position.y = circle1.modifiers[#stereocam_settings].focus
				Bracket.renderable = off
				bracket.showFrozenInGray = false
				Bracket.wirecolor = [22,76,157]
				setTransformLockFlags Bracket #{1..9}
				
				------------------------------------------
				-- NEARPLANE
				------------------------------------------
				
				NearPlane = plane length:50 width:50 name:(circle1.name+"_NearRange") isSelected:off pos:[0,0,0] widthsegs:2 lengthsegs:2
				NearPlane.transform = circle1.transform
				rotate NearPlane (angleaxis 90 [1,0,0])
				NearPlane.parent = circle1
				NearPlane.renderable = off
				NearPlane.wirecolor = [22,76,157]
				setTransformLockFlags NearPlane #{1,2,4..9}
				NearPlane.width.controller = float_script()
				NearPlane.length.controller = float_script()
				
				
				nearPlane_XScript = float_script()
				nearPlane_XScript.addTarget "v_camdist" circle1.modifiers[#stereocam_settings].camdist.controller --e
				nearPlane_XScript.addTarget "v_focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				nearPlane_XScript.addTarget "v_unitstopixels" circle1.modifiers[#stereocam_settings].v_unitstopixels.controller
				nearPlane_XScript.addTarget "nearlimit" circle1.modifiers[#stereocam_settings].nearlimit.controller
				nearPlane_XScript.addTarget "imagewidth" circle1.modifiers[#stereocam_settings].stereocam_controls[#imagewidth]
				str = "v_bracketdisparity = (nearlimit*renderwidth/imagewidth)/v_unitsToPixels \n"
				str += "endpos = ((v_camdist*v_focus)/(v_bracketdisparity+v_camdist)) \n"
				nearPlane_XScript.script = str
				nearPlane.position.controller.XPosition.controller = nearPlane_XScript
				
				enablesceneredraw()
				nearPlane_lengthScript = float_script()
				nearPlane_lengthScript.addTarget "camdist" circle1.modifiers[#stereocam_settings].camdist.controller
				nearPlane_lengthScript.addtarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				nearPlane_lengthScript.addtarget "fov" circle1.modifiers[#stereocam_settings].fov.controller --z
				nearPlane_lengthScript.addtarget "fov_offset" circle1.modifiers[#stereocam_settings].fov_offset.controller
				nearPlane_lengthScript.addtarget "dist" nearPlane.position.controller.XPosition.controller
				nearPlane_lengthScript.script = " real_Fov = fov-fov_offset \n"
				nearPlane_lengthScript.script += "offset = 100-(100*dist/focus) \n"
				nearPlane_lengthScript.script += "((tan(real_fov/2)*dist)*2)-((camdist)*(offset/100))"
				nearplane.length.controller = nearPlane_lengthScript	
				
				nearPlane_widthScript = float_script()
				nearPlane_widthScript.addtarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				nearPlane_widthScript.addtarget "fov" circle1.modifiers[#stereocam_settings].fov.controller --z
				nearPlane_widthScript.addtarget "dist" nearPlane.position.controller.XPosition.controller
				nearPlane_widthScript.script = "(((tan(fov/2)*(dist)))*2)/((renderwidth as float) / (renderheight as float))"
				nearplane.width.controller = nearPlane_widthScript	
				
				nearPlane.position.controller.XPosition.controller.value = 130
				
				
				------------------------------------------
				--FARRPLANE
				------------------------------------------
			
				farPlane = plane length:50 width:50 isSelected:off pos:[0,0,0] widthsegs:2 lengthsegs:2 name:(circle1.name+"_FarRange")
				farPlane.transform = circle1.transform
				rotate farPlane (angleaxis 90 [1,0,0])
				farPlane.parent = circle1
				farPlane.renderable = off
				farPlane.wirecolor = [22,76,157]
				setTransformLockFlags farPlane #{2..9}
				farPlane.width.controller = float_script()
				farPlane.length.controller = float_script()
				
				farPlane_XScript = float_script()
				farPlane_XScript.addTarget "v_camdist" circle1.modifiers[#stereocam_settings].camdist.controller --e
				farPlane_XScript.addTarget "v_focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				farPlane_XScript.addTarget "v_unitsToPixels" circle1.modifiers[#stereocam_settings].v_unitstopixels.controller
				farPlane_XScript.addTarget "farlimit" circle1.modifiers[#stereocam_settings].farlimit.controller
				farPlane_XScript.addTarget "imagewidth" circle1.modifiers[#stereocam_settings].stereocam_controls[#imagewidth]
				str = "v_bracketdisparity = (farlimit*renderwidth/imagewidth)/v_unitsToPixels \n"
				str += "endpos = -((v_camdist*v_focus)/(v_bracketdisparity-v_camdist)) \n"
				str += "if endpos > 0 then endpos else  3.40E5 \n"
				farPlane_XScript.script = str
				farPlane.position.controller.XPosition.controller = farPlane_XScript
				
				farPlane_lengthScript = float_script()
				farPlane_lengthScript.addtarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				farPlane_lengthScript.addtarget "fov" circle1.modifiers[#stereocam_settings].fov.controller --z
				farPlane_lengthScript.addtarget "dist" farPlane.position.controller.XPosition.controller
				farPlane_lengthScript.addtarget "fov_offset" circle1.modifiers[#stereocam_settings].fov_offset.controller
				farPlane_lengthScript.addTarget "camdist" circle1.modifiers[#stereocam_settings].camdist.controller
				farPlane_lengthScript.script = " real_Fov = fov-fov_offset \n"
				farPlane_lengthScript.script += "offset = 100-(100*dist/focus) \n"
				--farPlane_lengthScript.script += "((tan(fov/2)*(dist)))*2"
				farPlane_lengthScript.script += "((tan(real_fov/2)*dist)*2)+((camdist)*(offset/100))"
				farplane.length.controller = farPlane_lengthScript	
				
				farPlane_widthScript = float_script()
				farPlane_widthScript.addtarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				farPlane_widthScript.addtarget "fov" circle1.modifiers[#stereocam_settings].fov.controller --z
				farPlane_widthScript.addtarget "dist" farPlane.position.controller.XPosition.controller
				farPlane_widthScript.script = "(((tan(fov/2)*(dist)))*2)/((renderwidth as float) / (renderheight as float))"
				farplane.width.controller = farPlane_widthScript	
				
				
				------------------------------------------
				-- SCREENPLANE
				------------------------------------------	
					
				ScreenPlane = plane length:50 width:50 name:(circle1.name+"_ScreenPlane") isSelected:off pos:[0,0,0] widthsegs:2 lengthsegs:2
				ScreenPlane.transform = circle1.transform
				rotate ScreenPlane (angleaxis 90 [1,0,0])
				ScreenPlane.parent = circle1
				ScreenPlane.renderable = off
				ScreenPlane.wirecolor = [22,76,157]
				setTransformLockFlags ScreenPlane #{1,2,4..9}
				ScreenPlane.width.controller = float_script()
				ScreenPlane.length.controller = float_script()
				
				
				ScreenPlane_XScript = float_script()
				ScreenPlane_XScript.addTarget "camdist" circle1.modifiers[#stereocam_settings].camdist.controller --e
				ScreenPlane_XScript.addTarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				ScreenPlane_XScript.script = "focus"
				---ScreenPlane.position.controller.XPosition.controller = ScreenPlane_XScript
				ScreenPlane.position.controller.XPosition.controller = stereocam_focus.position.controller.Xposition.controller
				
				ScreenPlane_lengthScript = float_script()
				ScreenPlane_lengthScript.addtarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				ScreenPlane_lengthScript.addtarget "fov" circle1.modifiers[#stereocam_settings].fov.controller --z
				ScreenPlane_lengthScript.addtarget "dist" ScreenPlane.position.controller.XPosition.controller
				ScreenPlane_lengthScript.script = "((tan(fov/2)*(focus)))*2"
				ScreenPlane.length.controller = ScreenPlane_lengthScript	
				
				ScreenPlane_widthScript = float_script()
				ScreenPlane_widthScript.addtarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				ScreenPlane_widthScript.addtarget "fov" circle1.modifiers[#stereocam_settings].fov.controller --z
				ScreenPlane_widthScript.addtarget "dist" ScreenPlane.position.controller.XPosition.controller
				ScreenPlane_widthScript.script = "(((tan(fov/2)*(focus)))*2)/(getRendImageAspect())"
				ScreenPlane.width.controller = ScreenPlane_widthScript
				
					
				
				/*
				------------------------------------------
				-- DEPTHBRACKET dummies
				------------------------------------------
				
				bracket_startP = point name:(circle1.name+"_BracketStart") isSelected:off pos:[0,0,0] box:true
				bracket_startP.transform = circle1.transform
				bracket_startP.parent = circle1
				
				bracket_endP = point name:(circle1.name+"_BracketEnd") isSelected:off pos:[0,0,0] box:true
				bracket_endP.transform = circle1.transform
				bracket_endP.parent = circle1
				*/
				
				converttomesh bracket
				animateVertex bracket #all
				
				br_Far_script = point3_script()
				br_Far_script.addTarget "XPos" farPlane.position.controller.XPosition.controller
				br_Far_script.addTarget "length" farPlane.length.controller
				br_Far_script.addTarget "width" farPlane.width.controller
				
				br_screen_script = point3_script()
				br_screen_script.addTarget "focus" circle1.modifiers[#stereocam_settings].focus.controller 
				br_screen_script.addTarget "XPos" ScreenPlane.position.controller.XPosition.controller
				br_screen_script.addTarget "length" ScreenPlane.length.controller
				br_screen_script.addTarget "width" ScreenPlane.width.controller
				br_screen_script.addTarget "fXPos" farPlane.position.controller.XPosition.controller
				br_screen_script.addTarget "flength" farPlane.length.controller
				br_screen_script.addTarget "fwidth" farPlane.width.controller
				br_screen_script.addTarget "nXPos" NearPlane.position.controller.XPosition.controller
				br_screen_script.addTarget "nlength" NearPlane.length.controller
				br_screen_script.addTarget "nwidth" NearPlane.width.controller
				
				br_Near_script = point3_script()
				br_Near_script.addTarget "XPos" NearPlane.position.controller.XPosition.controller
				br_Near_script.addTarget "length" NearPlane.length.controller
				br_Near_script.addTarget "width" NearPlane.width.controller
				
				
				--FUL_script = "if nXPos <= sXPos then [(Xpos-thisXpos),-(width/2),(length/2+25)] else [(Xpos-thisXpos),-(width/2),(length/2+25)]"
				
				FUL_script = "[Xpos,-(width/2),(length/2+25)]"
				FUR_script = "[Xpos,-(width/2),-(length/2-25)]"
				FBL_script = "[Xpos,(width/2),(length/2+25)]"
				FBR_script = "[Xpos,(width/2),-(length/2-25)]"
				
				SUL_script = "if nXPos <= focus and fXPos >= focus then res = [Xpos,-(width/2),(length/2+25)] \n if nXpos > focus then res = [fXpos,-(fwidth/2),(flength/2+25)] \n if fXpos < focus then res = [nXpos,-(nwidth/2),(nlength/2+25)] \n res"
				SUR_script = "if nXPos <= focus and fXPos >= focus then res = [Xpos,-(width/2),-(length/2-25)] \n if nXpos > focus then res = [fXpos,-(fwidth/2),-(flength/2-25)] \n if fXpos < focus then res = [nXpos,-(nwidth/2),-(nlength/2-25)] \n res"
				SBL_script = "if nXPos <= focus and fXPos >= focus then res = [Xpos,(width/2),(length/2+25)] \n if nXpos > focus then res = [fXpos,(fwidth/2),(flength/2+25)] \n if fXpos < focus then res = [nXpos,(nwidth/2),(nlength/2+25)] \n res"
				SBR_script = "if nXPos <= focus and fXPos >= focus then res = [Xpos,(width/2),-(length/2-25)] \n if nXpos > focus then res = [fXpos,(fwidth/2),-(flength/2-25)] \n if fXpos < focus then res = [nXpos,(nwidth/2),-(nlength/2-25)] \n res"
				
				NUL_script = "[Xpos,-(width/2),(length/2+25)]"
				NUR_script = "[Xpos,-(width/2),-(length/2-25)]"
				NBL_script = "[Xpos,(width/2),(length/2+25)]"
				NBR_script = "[Xpos,(width/2),-(length/2-25)]"
				
				
				bracket.vertex9.controller = copy br_Far_script
				bracket.vertex9.controller.script = FUL_script
				bracket.vertex3.controller = copy br_Far_script
				bracket.vertex3.controller.script = FUR_script
				bracket.vertex12.controller = copy br_Far_script
				bracket.vertex12.controller.script = FBL_script
				bracket.vertex6.controller = copy br_Far_script
				bracket.vertex6.controller.script = FBR_script
				
				bracket.vertex8.controller = copy br_Screen_script
				bracket.vertex8.controller.script = SUL_script
				bracket.vertex2.controller = copy br_Screen_script
				bracket.vertex2.controller.script = SUR_script
				bracket.vertex11.controller = copy br_Screen_script
				bracket.vertex11.controller.script = SBL_script
				bracket.vertex5.controller = copy br_Screen_script
				bracket.vertex5.controller.script = SBR_script
				
				bracket.vertex7.controller = copy br_Near_script
				bracket.vertex7.controller.script = NUL_script 
				bracket.vertex1.controller = copy br_Near_script
				bracket.vertex1.controller.script = NUR_script 
				bracket.vertex10.controller = copy br_Near_script
				bracket.vertex10.controller.script = NBL_script 
				bracket.vertex4.controller = copy br_Near_script
				bracket.vertex4.controller.script = NBR_script 
				
				deleteface bracket 19
				deleteface bracket 19
				deleteface bracket 14
				deleteface bracket 13
				
				/**********************************************************************
				-- FLOATING stereoscopic FRAME (fsf)
				-- creates two planes with animated vertices. Each vertex has a point3_script
				-- defining it's xyz positons which represent: height, width and depth
				***********************************************************************/
				
				--fsf = floating stereoscopic frame
				fsf_L = plane name:(circle1.name+"_left_FSF") isSelected:off widthsegs:1 lengthsegs:1
				fsf_L.transform = circle1.transform
				rotate fsf_L (angleaxis 90 [1,0,0])
				fsf_L.parent = circle1
				convertToMesh fsf_L
				fsf_R = copy fsf_L name:(circle1.name+"_right_FSF")
				floatwindow = copy fsf_L name:(circle1.name+"_floatingWindow")
				fsf_L.castShadows = off
				fsf_L.receiveshadows = off
				fsf_L.ApplyAtmospherics = off
				fsf_L.secondaryVisibility = off
				fsf_L.wirecolor = [0,0,0]
				fsf_R.castShadows = off
				fsf_R.receiveshadows = off
				fsf_R.ApplyAtmospherics = off
				fsf_R.secondaryVisibility = off
				fsf_R.wirecolor = [0,0,0]
				floatwindow.castShadows = off
				floatwindow.receiveshadows = off
				floatwindow.ApplyAtmospherics = off
				floatwindow.secondaryVisibility = off
				floatwindow.wirecolor = [0,0,0]
				floatwindow.visibility = 0.2
				solidblackmaterial = Standardmaterial ()
				solidblackmaterial.Diffuse = color 0 0 0
				solidblackmaterial.Ambient = color 0 0 0
				solidblackmaterial.glossiness = 0
				solidblackmaterial.selfIllumAmount = 100
				solidblackmaterial.twoSided = true 
				fsf_L.material = solidblackmaterial
				fsf_R.material = solidblackmaterial
				floatwindow.material = solidblackmaterial
				solidblackmaterial.diffuse.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.frameColor.controller
				animateVertex fsf_L #all
				animateVertex fsf_R #all
				animateVertex floatwindow #all

				fsf_vpos_scr = point3_script()
				fsf_vpos_scr.addTarget "camdist" circle1.modifiers[#stereocam_settings].camdist.controller --e
				fsf_vpos_scr.addTarget "focus" circle1.modifiers[#stereocam_settings].focus.controller --z
				fsf_vpos_scr.addtarget "fov_offset" circle1.modifiers[#stereocam_settings].fov_offset.controller
				fsf_vpos_scr.addtarget "fov" circle1.modifiers[#stereocam_settings].fov.controller --z
				fsf_vpos_scr.addTarget "imagewidth_offset" circle1.modifiers[#stereocam_settings].imagewidth_offset.controller
				
				fsf_str = " real_Fov = fov-fov_offset \n"
				fsf_str += "real_imagewidth = renderwidth-imagewidth_offset \n"
				fsf_str += "ffov = 2*atan((renderwidth*tan(real_fov/2))/real_imagewidth) \n"
				fsf_str += " posZ =  -focus+(focus*(offset/100)) \n"  -- distance to stereocam 
				fsf_str += " hw =  tan(real_fov/2)*posZ-(camdist*(-(offset-100)/200)) \n " --half width + offset 
				fsf_str += " height =  ((((tan(ffov/2)*posZ))*2)/(getrendImageaspect()))/2\n" -- half height
				fsf_str += " width =   camdist/2  \n" -- + ( ((tan(fov/2)*(focus)))/(getRendImageAspect()) )\n"
				
				fsf_str_LUL = fsf_str + "[-height,width-hw,posZ]"
				fsf_str_LUR = fsf_str + "[-height,-width-hw,posZ]"
				fsf_str_LBL = fsf_str + "[height,width-hw,posZ]"
				fsf_str_LBR = fsf_str + "[height,-width-hw,posZ]"
				
				fsf_str_RUL = fsf_str + "[-height,width+hw,posZ]"
				fsf_str_RUR = fsf_str + "[-height,-width+hw,posZ]"
				fsf_str_RBL = fsf_str + "[height,width+hw,posZ]"
				fsf_str_RBR = fsf_str + "[height,-width+hw,posZ]"
				
				fsf_LUL_scr = copy fsf_vpos_scr
				fsf_LUL_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].UpLeft.controller
				fsf_LUL_scr.script = fsf_str_LUL
				fsf_LUR_scr = copy fsf_vpos_scr
				fsf_LUR_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].UpLeft.controller
				fsf_LUR_scr.script = fsf_str_LUR
				fsf_LBL_scr = copy fsf_vpos_scr
				fsf_LBL_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].DownLeft.controller
				fsf_LBL_scr.script = fsf_str_LBL
				fsf_LBR_scr = copy fsf_vpos_scr
				fsf_LBR_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].DownLeft.controller
				fsf_LBR_scr.script = fsf_str_LBR
				
				fsf_RUL_scr = copy fsf_vpos_scr
				fsf_RUL_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].UpRight.controller
				fsf_RUL_scr.script = fsf_str_RUL
				fsf_RUR_scr = copy fsf_vpos_scr
				fsf_RUR_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].UpRight.controller
				fsf_RUR_scr.script = fsf_str_RUR
				fsf_RBL_scr = copy fsf_vpos_scr
				fsf_RBL_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].DownRight.controller
				fsf_RBL_scr.script = fsf_str_RBL
				fsf_RBR_scr = copy fsf_vpos_scr
				fsf_RBR_scr.addTarget "offset" circle1.modifiers[#stereocam_settings].DownRight.controller
				fsf_RBR_scr.script = fsf_str_RBR
				
				fsf_L.vertex1.controller = fsf_LBR_scr
				fsf_L.vertex2.controller = fsf_LBL_scr
				fsf_L.vertex3.controller = fsf_LUR_scr
				fsf_L.vertex4.controller = fsf_LUL_scr
				
				fsf_R.vertex1.controller = fsf_RBR_scr
				fsf_R.vertex2.controller = fsf_RBL_scr
				fsf_R.vertex3.controller = fsf_RUR_scr
				fsf_R.vertex4.controller = fsf_RUL_scr
				
				floatwindow.vertex1.controller = fsf_RBL_scr
				floatwindow.vertex2.controller = fsf_RUL_scr
				floatwindow.vertex3.controller = fsf_LBR_scr
				floatwindow.vertex4.controller = fsf_LUR_scr
				
				
				
			/****************************************************************************************************************************/
			/*   create and assign materials
			/****************************************************************************************************************************/
					
				nearmaterial = Standardmaterial ()
				nearmaterial.Diffuse = color 88 144 255 
				nearmaterial.Ambient = color 88 144 255 
				nearmaterial.glossiness = 0
				nearmaterial.selfIllumAmount = 100
				nearPlane.material = nearmaterial
				nearmaterial.diffuse.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.nearColor.controller
				nearmaterial.opacity.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.nearOpacity.controller
				NearPlane.wirecolor = nearmaterial.Diffuse
				
				farmaterial = Standardmaterial ()
				farmaterial.Diffuse = color 88 144 255 
				farmaterial.Ambient = color 88 144 255 
				farmaterial.glossiness = 0
				farmaterial.selfIllumAmount = 100
				farPlane.material = farmaterial
				farmaterial.diffuse.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.farColor.controller
				farmaterial.opacity.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.farOpacity.controller
				FarPlane.wirecolor = Farmaterial.Diffuse
				
				screenmaterial = Standardmaterial ()
				screenmaterial.Diffuse = color 88 144 255 
				screenmaterial.Ambient = color 88 144 255 
				screenmaterial.glossiness = 0
				screenmaterial.selfIllumAmount = 100
				screenPlane.material = screenmaterial
				screenmaterial.diffuse.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.screenColor.controller
				screenmaterial.opacity.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.screenOpacity.controller
				screenPlane.wirecolor = screenmaterial.Diffuse
				
				bracketmaterial = Standardmaterial ()
				bracketmaterial.Diffuse = color 88 144 255 
				bracketmaterial.Ambient = color 88 144 255 
				bracketmaterial.glossiness = 0
				bracketmaterial.selfIllumAmount = 100
				bracket.material = bracketmaterial
				bracketmaterial.diffuse.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.volumeColor.controller
				bracketmaterial.opacity.controller = circle1.modifiers[#stereoCam_settings].stereoCam_Controls.volumeOpacity.controller
				bracket.wirecolor = bracketmaterial.Diffuse
				
				
			/****************************************************************************************************************************/
			/*   create and rig cameras
			/****************************************************************************************************************************/

			circle1.rotation.controller = rotation_list()
			if classof referencecam == Targetcamera or referencecam == undefined or classof referencecam == VrayPhysicalcamera then
			(
				lk = LookAt_Constraint ()
				lk.appendTarget stereoCam_lookAt 100
				lk.viewline_length_abs = false
				lk.target_Axis = 0
				lk.upnode_world = on
				lk.upnode_ctrl = 1
				circle1.rotation.controller[1].controller = lk	
				circle1.rotation.controller[#available].controller = euler_xyz()
				circle1.rotation.controller.active = 2
				circle1.rotation.controller.setname 1 "lookat"
				circle1.rotation.controller.setname 2 "roll"
				lk = float_limit()
				lk.lower_limit = 0;
				lk.upper_limit = 0;
				circle1.rotation.controller[2].controller[2].controller = lk
				circle1.rotation.controller[2].controller[3].controller = lk
			)
			else if classof referenceCam == FreeCamera then
			(
				circle1.rotation.controller.setname 1 "roll"
			)			
			paramWire.connect circle1.modifiers[#stereoCam_settings].stereoCam_Controls[#focus] circle1.modifiers[#XForm].Gizmo[#scale] "[focus/50,focus/50,focus/50]"
			
			if classof referenceCam != freecamera do paramWire.connect circle1.modifiers[#stereoCam_settings].stereoCam_Controls[#focus] stereoCam_lookAt.baseObject[#Size] "focus/10"
			paramWire.connect circle1.modifiers[#stereoCam_settings].stereoCam_Controls[#focus] stereoCam_focus.baseObject[#Size] "focus/5"


			if classof referencecam != VrayPhysicalCamera then
			(
				stereoCam_camera = Targetcamera fov:45 nearclip:1 farclip:1000 nearrange:0 farrange:1000 mpassEnabled:off mpassRenderPerPass:off pos:[0,0,0] isSelected:off target:(Targetobject parent:stereoCam_focus transform:(matrix3 [1,0,0] [0,1,0] [0,0,1] [stereoCam_focus.pos.x,stereoCam_focus.pos.y,stereoCam_focus.pos.z]))
				stereoCam_camera.fov.controller =  circle1.modifiers[#stereocam_settings].fov.controller
				stereoCam_camera.nearClip.controller = NearPlane.position.controller.Xposition.controller
				stereoCam_camera.FarClip.controller = FarPlane.position.controller.Xposition.controller
			)
			else 
			(
				stereoCam_camera = VrayPhysicalCamera pos:[0,0,0] isSelected:off target:(Targetobject parent:stereoCam_focus transform:(matrix3 [1,0,0] [0,1,0] [0,0,1] [stereoCam_focus.pos.x,stereoCam_focus.pos.y,stereoCam_focus.pos.z]))
				stereoCam_camera.focal_length.controller = float_script()
				stereoCam_camera.Clip_near.controller = NearPlane.position.controller.Xposition.controller
				stereoCam_camera.Clip_far.controller = FarPlane.position.controller.Xposition.controller	
					
				stereoCam_camera.zoom_factor.controller = bezier_float()
				stereoCam_camera.focus_distance.controller = bezier_float()
				stereoCam_camera.film_width.controller = bezier_float()
				stereoCam_camera.target_distance.controller = bezier_float()
				stereoCam_camera.specify_focus.controller = boolean_float()	
					
			)			
			
			--circle1.modifiers[#stereocam_settings].bracketClipping.controller = boolean_float()
			--stereoCam_camera.clipManually.controller = circle1.modifiers[#stereocam_settings].bracketClipping.controller	
			
				
			
			
				
			pos_script_L1 = float_script()
			pos_script_L1.addTarget "distance" circle1.modifiers[#stereocam_settings].camdist.controller
			pos_script_L1.script = "(0.5*distance)"
			--if classof referencecam == VrayPhysicalcamera then	NewStereoCam_L1 = copy stereoCam_camera name:(circle1.name+"_left3dCam")
			NewStereoCam_L1 = reference stereoCam_camera name:(circle1.name+"_left3dCam")
			NewStereoCam_L1.target = reference stereoCam_camera.target
			NewStereoCam_L1.parent = circle1	
			NewStereoCam_L1.position.controller.YPosition.controller = pos_script_L1
			--if classof referenceCam == VrayPhysicalCamera do NewStereoCam_L1.target.position.controller.YPosition.controller = pos_script_L1	
			NewStereoCam_L1.wirecolor = [88,144,255]
			--paramWire.connect circle1.rotation.controller.Euler_XYZ.controller[#X_Rotation] NewStereoCam_L1.transform.controller[#Roll_Angle] "-X_Rotation"	
			NewStereoCam_L1.controller.UseTargetAsUpNode = true
			if classof referenceCam != VrayPhysicalCamera then
			(
				skew_L1 = skew()
				skew_L1. axis = 1
				skew_L1.amount = 0.298
				addModifier NewStereoCam_L1 (skew_L1)
				skew_amount_script_L1 = float_script()
				skew_amount_script_L1.addTarget "tdist" circle1.modifiers[#stereocam_settings].focus.controller
				skew_amount_script_L1.addTarget "CamX" circle1.modifiers[#stereocam_settings].camdist.controller
				skew_amount_script_L1.addTarget "convergence" circle1.modifiers[#stereocam_settings].convergence.controller
				--skew_amount_script_L1.addTarget "offset" circle1.modifiers[#stereocam_settings].bracketoffsetDisp.controller
				skew_amount_script_L1.addTarget "unitstopixels" circle1.modifiers[#stereocam_settings].v_unitsToPixels.controller
				skew_amount_script_L1.addNode "thisCamera" NewStereoCam_L1
				str = "MaxSkew = getModContextBBoxMax ThisCamera ThisCamera.modifiers[1] \n"
				str += "MinSkew = getModContextBBoxMin ThisCamera ThisCamera.modifiers[1] \n"
				str += "Xmax = MaxSkew.x \n"
				str += "XMin = MinSkew.x \n"
				str += "skewamount = ((thiscamera.position.controller.YPosition.controller.value)/(tdist))*(Xmax-Xmin) \n"
				str += "result = abs(convergence-1)*skewamount"
				skew_amount_script_L1.script = str
				skew_L1.amount.controller = skew_amount_script_L1
				--paramWire.connect circle1.modifiers[#stereocam_settings].camdist.controller NewStereoCam_L1.target.transform.controller.position.controller[#XPosition] "-X_Rotation"	
				--paramWire.connect circle1.modifiers[#stereoCam_settings].stereoCam_Controls[#bracketoffsetDisp] NewStereoCam_L1.target.pos.controller[#X_Position] "bracketoffsetDisp"
			)
				
			pos_script_R1 = float_script()
			pos_script_R1.addTarget "distance" circle1.modifiers[#stereocam_settings].camdist.controller
			pos_script_R1.script = "-(0.5*distance)"
			--if classof referencecam == VrayPhysicalcamera then NewStereoCam_R1 = copy stereoCam_camera name:(circle1.name+"_right3dCam")
			NewStereoCam_R1 = reference stereoCam_camera name:(circle1.name+"_right3dCam")
			NewStereoCam_R1.target = reference stereoCam_camera.target
			NewStereoCam_R1.parent = circle1	
			NewStereoCam_R1.position.controller.YPosition.controller = pos_script_R1
			--if classof referenceCam == VrayPhysicalCamera do NewStereoCam_R1.target.position.controller.YPosition.controller = pos_script_R1	
			NewStereoCam_R1.wirecolor = [88,144,255]
			--paramWire.connect circle1.rotation.controller.Euler_XYZ.controller[#X_Rotation] NewStereoCam_R1.transform.controller[#Roll_Angle] "-X_Rotation"	
			NewStereoCam_R1.controller.UseTargetAsUpNode = true
			if classof referenceCam != VrayPhysicalCamera then
			(
							
				skew_R1 = skew()
				skew_R1. axis = 1
				skew_R1.amount = 0.298
				addModifier NewStereoCam_R1 (skew_R1)
				skew_amount_script_R1 = float_script()
				skew_amount_script_R1.addTarget "tdist" circle1.modifiers[#stereocam_settings].focus.controller
				skew_amount_script_R1.addTarget "CamX" circle1.modifiers[#stereocam_settings].camdist.controller
				skew_amount_script_R1.addTarget "convergence" circle1.modifiers[#stereocam_settings].convergence.controller
				--skew_amount_script_R1.addTarget "offset" circle1.modifiers[#stereocam_settings].bracketoffsetDisp.controller
				skew_amount_script_R1.addTarget "unitstopixels" circle1.modifiers[#stereocam_settings].v_unitsToPixels.controller
				skew_amount_script_R1.addNode "thisCamera" NewStereoCam_R1
				str = "MaxSkew = getModContextBBoxMax ThisCamera ThisCamera.modifiers[1] \n"
				str += "MinSkew = getModContextBBoxMin ThisCamera ThisCamera.modifiers[1] \n"
				str += "Xmax = MaxSkew.x \n"
				str += "XMin = MinSkew.x \n"
				str += "skewamount = tan(atan(((thiscamera.position.controller.YPosition.controller.value)/(tdist))))*(Xmax-Xmin) \n"
				str += "result = abs(convergence-1)*skewamount"
				skew_amount_script_R1.script = str
				skew_R1.amount.controller = skew_amount_script_R1
			
				--paramWire.connect circle1.modifiers[#stereoCam_settings].stereoCam_Controls[#bracketoffsetDisp] NewStereoCam_R1.target.pos.controller[#X_Position] "bracketoffsetDisp"
				
				newFov_script_R1 = float_script()
				newFov_script_R1.addTarget "origtdist" circle1.modifiers[#stereocam_settings].focus.controller
				newFov_script_R1.addTarget "origfov" circle1.modifiers[#stereocam_settings].fov.controller
				newFov_script_R1.addTarget "convergence" circle1.modifiers[#stereocam_settings].convergence.controller
				--newFov_script_R1.addTarget "offset" circle1.modifiers[#stereocam_settings].bracketoffsetDisp.controller
				newFov_script_R1.addNode "thisCamera" NewStereoCam_R1
				str = "origwidth = 2*(tan(origfov/2)*(origtdist)) \n"
				str += "newFov = 2*atan(origwidth/(2*thiscamera.targetdistance)) \n"
				str += "result = ((degtorad newFov)*abs(convergence-1))+((degtorad origfov)*(1-abs(convergence-1))) "
				newFov_script_R1.script = str
				NewStereoCam_R1.fov.controller = newFov_script_R1
			)
			
			stereoCam_03 = reference stereoCam_camera name:(circle1.name+"_Center3dCam")
			stereoCam_03.target = reference stereoCam_camera.target 
			stereoCam_03.parent = circle1	
			stereoCam_03.wirecolor = [88,144,255]
			stereoCam_03.controller.UseTargetAsUpNode = true
			--try (paramWire.connect circle1.rotation.controller[#roll].controller[#X_Rotation] stereoCam_03.transform.controller[#Roll_Angle] "-X_Rotation") catch (print "n")
			--paramWire.connect circle1.modifiers[#stereoCam_settings].stereoCam_Controls[#bracketoffsetDisp] stereoCam_03.target.pos.controller[#X_Position] "bracketoffsetDisp"
			
			if classof referenceCam == VrayPhysicalCamera then -- Instanciate all controllers
			(
				/*cam1 = NewStereoCam_L1
				cam2 = NewStereoCam_R1
				cam3 = stereoCam_03
				cam_params = (getPropNames cam1)

				for i = 1 to cam_params.count do
				(
					--print cam_params[i]
					param_name = cam_params[i] as string
					
					if param_name != "clip_near" and param_name != "clip_far" then
					(
							
						
						param_type = classof (execute ("$"+cam1.name +"."+(cam_params[i] as string)))
						
						case param_type of
						(
							Float: newController = bezier_float()	
							BooleanClass: newController = Boolean_float()
							Integer: newController = bezier_float()	
							Color: newController = bezier_color()
						)		
						
						try ( (execute ("$"+cam1.name +".baseobject[#"+param_name+"]")).controller = newcontroller ) catch()
						try ( (execute ("$"+cam2.name +".baseobject[#"+param_name+"]")).controller = newcontroller ) catch()	
						try ( (execute ("$"+cam3.name +".baseobject[#"+param_name+"]")).controller = newcontroller ) catch()
					)
				)
				*/
				-- SET UP LEFT CONTROLLERS
				
				ca = attributes StereoAttributes 
				(
					parameters stereo_params rollout:stereo_rl
					(
						side type:#integer ui:(sidebuttons_rb) default:1
					)
					
					rollout stereo_rl "stereo attributes"
					(
						radiobuttons sidebuttons_rb "side" labels:#("left","center","right")  default:2 columns:3
					)
				)				
				custattributes.add NewStereoCam_L1.baseobject ca
				
				NewStereoCam_L1.baseobject.side.controller = bezier_float()
				
						
				
				skew_amount_script_L1 = float_script()
				skew_amount_script_L1.addTarget "tdist" circle1.modifiers[#stereocam_settings].focus.controller
				skew_amount_script_L1.addTarget "CamX" circle1.modifiers[#stereocam_settings].camdist.controller
				skew_amount_script_L1.addTarget "convergence" circle1.modifiers[#stereocam_settings].convergence.controller
				skew_amount_script_L1.addTarget "side" NewStereoCam_L1.baseobject.side.controller
				str = "skewamount = -(side-2)*tan(atan(((camX)/(tdist))))*(0.02)*25 \n"
				str += "result = abs(convergence-1)*skewamount"
				skew_amount_script_L1.script = str	
				NewStereoCam_L1.lens_horShift.controller = skew_amount_script_L1
					
				newFov_script_L1 = float_script()
				newFov_script_L1.addTarget "origtdist" circle1.modifiers[#stereocam_settings].focus.controller
				newFov_script_L1.addTarget "origfov" circle1.modifiers[#stereocam_settings].fov.controller
				newFov_script_L1.addObject "mag" NewStereoCam_L1.baseobject.zoom_factor.controller
				newFov_script_L1.addObject "focus" NewStereoCam_L1.baseobject.focus_distance.controller
				newFov_script_L1.addObject "gate" NewStereoCam_L1.baseobject.film_width.controller
				newFov_script_L1.addObject "targetdistance" NewStereoCam_L1.baseobject.target_distance.controller
				newFov_script_L1.addObject "specify_focus" NewStereoCam_L1.baseobject.specify_focus.controller
				--newFov_script_L1.addTarget "convergence" circle1.modifiers[#stereocam_settings].convergence.controller
				--newFov_script_L1.addTarget "offset" circle1.modifiers[#stereocam_settings].bracketoffsetDisp.controller
				--newFov_script_L1.addNode "thisCamera" NewStereoCam_L1
				str = "fdist = (focus.value*specify_focus.value)+(targetdistance.value*(abs (specify_focus.value-1))) \n "
				str += "newFov = 2*atan((origtdist*tan(origfov/2))/(targetdistance.value)) \n"
				str += "focal = gate.value/(2*mag.value*tan(newfov/2)) \n"
				str += "result = ((fdist*10)*focal)/((fdist*10)+focal) \n"
				--str += "newFov = 2*atan((origtdist*tan(origfov/2))/(targetdistance.value)) \n"
				--str += "result = camerafov.fovtomm newFov"
				newFov_script_L1.script = str
				NewStereoCam_L1.focal_length.controller = newFov_script_L1	
				
				
				-- SET UP RIGHT CONTROLLERS
				
				/*
				skew_amount_script_R1 = float_script()
				skew_amount_script_R1.addTarget "tdist" circle1.modifiers[#stereocam_settings].focus.controller
				skew_amount_script_R1.addTarget "CamX" circle1.modifiers[#stereocam_settings].camdist.controller
				skew_amount_script_R1.addTarget "convergence" circle1.modifiers[#stereocam_settings].convergence.controller
				str = "skewamount = -tan(atan(((camX)/(tdist))))*(0.02)*25 \n"
				str += "result = abs(convergence-1)*skewamount"
				skew_amount_script_R1.script = str
				NewStereoCam_R1.lens_horShift.controller = skew_amount_script_R1
					
				NewStereoCam_R1.focal_length.controller = NewStereoCam_L1.focal_length.controller
				*/
				
				-- SET UP CENTER CONTROLLERS
				/*
				newFov_script_Center = float_script()
				--newFov_script_Center.addTarget "origtdist" circle1.modifiers[#stereocam_settings].focus.controller
				newFov_script_Center.addTarget "origfov" circle1.modifiers[#stereocam_settings].fov.controller
				newFov_script_Center.addObject "mag" stereoCam_03.baseobject.zoom_factor.controller
				newFov_script_Center.addObject "focus" stereoCam_03.baseobject.focus_distance.controller
				newFov_script_Center.addObject "gate" stereoCam_03.baseobject.film_width.controller
				newFov_script_Center.addObject "targetdistance" stereoCam_03.baseobject.target_distance.controller
				newFov_script_Center.addObject "specify_focus" stereoCam_03.baseobject.specify_focus.controller
				--newFov_script_L1.addTarget "convergence" circle1.modifiers[#stereocam_settings].convergence.controller
				--newFov_script_L1.addTarget "offset" circle1.modifiers[#stereocam_settings].bracketoffsetDisp.controller
				--newFov_script_Center.addNode "thisCamera" NewStereoCam_L1
				--str = "newFov = 2*atan((origtdist*tan(origfov/2))/(targetdistance.value)) \n"
				str = "fdist = (focus.value*specify_focus.value)+(targetdistance.value*(abs (specify_focus.value-1))) \n "
				str += "focal = gate.value/(2*mag.value*tan(origfov/2)) \n"
				str += "result = ((focus.value*10)*focal)/((focus.value*10)+focal) \n"
				--str += "newFov = 2*atan((origtdist*tan(origfov/2))/(targetdistance.value)) \n"
				--str += "result = camerafov.fovtomm newFov"
				newFov_script_Center.script = str
				stereoCam_03.focal_length.controller = newFov_script_Center	
				*/
			)
			
			delete stereoCam_camera
				
				
				
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.camera_nodes = #(NewStereoCam_L1.name,stereoCam_03.name,NewStereoCam_R1.name,stereoCam_helper.name)
			if classof referenceCam != FreeCamera then
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.helper_nodes = #(FarPlane.name, screenPlane.name, nearPlane.name, fsf_L.name, fsf_r.name, floatwindow.name, stereoCam_focus.name, stereocam_lookat.name, bracket.name)	
			else
			circle1.modifiers[#stereocam_settings].stereoCam_Controls.helper_nodes = #(FarPlane.name, screenPlane.name, nearPlane.name, fsf_L.name, fsf_r.name, floatwindow.name, stereoCam_focus.name, "undefined", bracket.name)	
			
			append circle1.modifiers[#stereocam_settings].stereo_Layers circle1.name
				
			--paramWire.connect2way circle1.modifiers[#stereoCam_settings].stereoCam_Controls[#fov] stereoCam_02.baseObject[#FOV] "FOV" "fov"
			--circle1.modifiers[#stereoCam_settings].stereoCam_Controls.fov.controller = stereoCam_02.baseObject[#FOV] "FOV" "fov"
			
			hide #(floatwindow,fsf_R, fsf_L, stereocam_focus)
			for element in #(floatwindow,fsf_R, fsf_L, nearPlane, screenPlane, farPlane) do element.showfrozeningray = false
			freeze #(floatwindow,fsf_R, fsf_L)
			
			if (LayerManager.getLayerFromName circle1.name == undefined) then (camlayer = LayerManager.newLayerFromName circle1.name) else (camlayer = LayerManager.newLayerFromName (circle1.name+"_layer2"))
			try(
			for i = 1 to (circle1.children.count) do camlayer.addnode circle1.children[i]
			camlayer.addnode circle1
			camlayer.addnode stereoCam_lookat
			camlayer.addnode nearRange
			camlayer.addnode farRange
			--addModifier stereoCam_03 (circle1.modifiers[#stereocam_settings])
			--hide stereoCam_02
			--hide stereoCam_04
			--hide circle1		
			) catch()

			
			if (referenceCam != undefined) then 
			(
				
				circle1.pos.controller = referenceCam.pos.controller 
				if Classof referenceCam == TargetCamera then 
				(
					stereoCam_lookAt.pos.controller = referenceCam.target.pos.controller
					circle1.modifiers[#stereocam_settings].stereoCam_Controls.focus = referenceCam.targetdistance
					circle1.rotation.controller[#roll].controller[1].controller = referenceCam.transform.controller.roll_angle.controller
					--circle1.rotation.controller[#available].controller = euler_xyz()
					--circle1.rotation.controller.setname 3 "correction"
					--paramWire.connect circle1.rotation.controller.roll.controller[#X_Rotation] circle1.rotation.controller.correction.controller[#X_Rotation] "-X_Rotation*2"
					circle1.modifiers[#stereocam_settings].stereoCam_Controls.fov = referenceCam.FOV
				)	
				else if Classof referenceCam == FreeCamera then
				(
					
					circle1.rotation.controller[#roll].controller = referenceCam.rotation.controller
					circle1.rotation.controller.setname 1 "roll"
					
					
					circle1.rotation.controller[#available].controller = euler_xyz()
					circle1.rotation.controller.setname 2 "correction"
					circle1.rotation.controller.setactive 2
					circle1.rotation.controller[#correction].controller.Y_Rotation = 90
					circle1.rotation.controller[#correction].controller.Z_Rotation = 90
					--rotate circle1 (angleaxis -90 [0,1,0])
					--rotate circle1 (angleaxis -90 [1,0,0])
					circle1.rotation.controller.setactive 1
					circle1.modifiers[#stereocam_settings].stereoCam_Controls.focus = referenceCam.target_distance
					circle1.modifiers[#stereocam_settings].stereoCam_Controls.fov = referenceCam.FOV
				)
				else if Classof referenceCam == VrayPhysicalCamera then
				(
					stereoCam_lookAt.pos.controller = referenceCam.target.pos.controller
					circle1.modifiers[#stereocam_settings].stereoCam_Controls.focus = referenceCam.focus_distance
					circle1.rotation.controller[#roll].controller[1].controller = referenceCam.transform.controller.roll_angle.controller
				)
				--rightCam.visible = false
				--delete referenceCam
			)

			
			try (append allStereoCameras circle1) 
			catch 
			(
				allStereoCameras = #()
				append allStereoCameras circle1
			)
			
		)	
		undo on


		enablesceneredraw()
		forcecompleteredraw()
	--)


--) -- END of MACRO
